@@ -47,6 +47,10 @@
 static unsigned long timer_slack = 0;
 #endif
 
+#if defined(__linux__) && defined(HAVE_SYS_CAP_H)
+#include <sys/capability.h>
+#endif
+
 static const char *stress_temp_path = ".";
 
 /*
@@ -222,13 +226,29 @@ uint64_t stress_get_filesystem_size(void)
 	if (blocks > max_blocks)
 		blocks = max_blocks;
 
-	(void)printf("stress_get_filesystem_size: %" PRIu64 " %" PRIu64 " %" PRIu64 "\n",
-		(uint64_t)buf.f_bsize, (uint64_t)blocks,
-		(uint64_t)buf.f_bsize * blocks);
-
 	return (uint64_t)buf.f_bsize * blocks;
 }
 
+/*
+ *  stress_get_filesystem_available_inodes()
+ *	get number of free available inodes on the current stress
+ *	temporary path, return 0 if failed
+ */
+uint64_t stress_get_filesystem_available_inodes(void)
+{
+	int rc;
+	struct statvfs buf;
+
+	if (!stress_temp_path)
+		return 0;
+
+	rc = statvfs(stress_temp_path, &buf);
+	if (rc < 0)
+		return 0;
+
+	return (uint64_t)buf.f_favail;
+}
+
 /*
  *  stress_set_nonblock()
  *	try to make fd non-blocking
@@ -533,8 +553,8 @@ int stress_temp_dir_mk(
 	ret = mkdir(tmp, S_IRWXU);
 	if (ret < 0) {
 		ret = -errno;
-		pr_fail("%s: mkdir failed, errno=%d (%s)\n",
-			name, errno, strerror(errno));
+		pr_fail("%s: mkdir '%s' failed, errno=%d (%s)\n",
+			name, tmp, errno, strerror(errno));
 	}
 
 	return ret;
@@ -565,8 +585,8 @@ int stress_temp_dir_rm(
 	ret = rmdir(tmp);
 	if (ret < 0) {
 		ret = -errno;
-		pr_fail("%s: rmdir failed, errno=%d (%s)\n",
-			name, errno, strerror(errno));
+		pr_fail("%s: rmdir '%s' failed, errno=%d (%s)\n",
+			name, tmp, errno, strerror(errno));
 	}
 
 	return ret;
@@ -826,7 +846,7 @@ int system_read(
 	int fd;
 	ssize_t ret;
 
-	memset(buf, 0, buf_len);
+	(void)memset(buf, 0, buf_len);
 
 	fd = open(path, O_RDONLY);
 	if (fd < 0)
@@ -964,9 +984,9 @@ int stress_sighandler(
 	}
 #endif
 
-	memset(&new_action, 0, sizeof new_action);
+	(void)memset(&new_action, 0, sizeof new_action);
 	new_action.sa_handler = handler;
-	sigemptyset(&new_action.sa_mask);
+	(void)sigemptyset(&new_action.sa_mask);
 	new_action.sa_flags = SA_ONSTACK;
 
 	if (sigaction(signum, &new_action, orig_action) < 0) {
@@ -977,6 +997,27 @@ int stress_sighandler(
 	return 0;
 }
 
+/*
+ *  stress_handle_stop_stressing()
+ *	set flag to indicate to stressor to stop stressing
+ */
+void stress_handle_stop_stressing(int dummy)
+{
+	(void)dummy;
+
+	g_keep_stressing_flag = false;
+}
+
+/*
+ *  stress_sig_stop_stressing()
+ *	install a handler that sets the global flag
+ *	to indicate to a stressor to stop stressing
+ */
+int stress_sig_stop_stressing(const char *name, const int sig)
+{
+	return stress_sighandler(name, sig, stress_handle_stop_stressing, NULL);
+}
+
 /*
  *  stress_sigrestore()
  *	restore a handler
@@ -1172,3 +1213,68 @@ char *stress_uint64_to_str(char *str, size_t len, const uint64_t val)
 
 	return str;
 }
+
+#if defined(__linux__) && defined(HAVE_SYS_CAP_H)
+int stress_drop_capabilities(const char *name)
+{
+	int ret;
+	uint32_t i;
+	struct __user_cap_header_struct uch;
+	struct __user_cap_data_struct ucd[_LINUX_CAPABILITY_U32S_3];
+
+	(void)memset(&uch, 0, sizeof uch);
+	(void)memset(ucd, 0, sizeof ucd);
+
+	uch.version = _LINUX_CAPABILITY_VERSION_3;
+	uch.pid = getpid();
+
+	ret = capget(&uch, ucd);
+	if (ret < 0) {
+		pr_fail("%s: capget on pid %d failed: "
+			"errno=%d (%s)\n",
+			name, uch.pid, errno, strerror(errno));
+		return -1;
+	}
+
+	/*
+	 *  We could just memset ucd to zero, but
+	 *  lets explicity set all the capability
+	 *  bits to zero to show the intent
+	 */
+	for (i = 0; i <= CAP_LAST_CAP; i++) {
+		uint32_t idx = CAP_TO_INDEX(i);
+		uint32_t mask = CAP_TO_MASK(i);
+
+		ucd[idx].inheritable &= ~mask;
+		ucd[idx].permitted &= ~mask;
+		ucd[idx].effective &= ~mask;
+	}
+
+	ret = capset(&uch, ucd);
+	if (ret < 0) {
+		pr_fail("%s: capset on pid %d failed: "
+			"errno=%d (%s)\n",
+			name, uch.pid, errno, strerror(errno));
+		return -1;
+	}
+#if defined(PR_SET_NO_NEW_PRIVS)
+	ret = prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0);
+	if (ret < 0) {
+		pr_inf("%s: prctl PR_SET_NO_NEW_PRIVS on pid %d failed: "
+			"errno=%d (%s)\n",
+			name, uch.pid, errno, strerror(errno));
+		return -1;
+	}
+#endif
+
+
+	return 0;
+}
+#else
+int stress_drop_capabilities(const char *name)
+{
+	(void)name;
+
+	return 0;
+}
+#endif
