@@ -48,12 +48,13 @@ int pr_yaml(FILE *fp, const char *const fmt, ...)
 
 /*
  *  pr_closelog()
- *	log closing, performed via exit()
+ *	log closing
  */
-static void pr_closelog(void)
+void pr_closelog(void)
 {
 	if (log_file) {
-		fclose(log_file);
+		(void)fflush(log_file);
+		(void)fclose(log_file);
 		log_file = NULL;
 	}
 }
@@ -69,10 +70,9 @@ void pr_openlog(const char *filename)
 
 	log_file = fopen(filename, "w");
 	if (!log_file) {
-		pr_err(stderr, "Cannot open log file %s\n", filename);
+		pr_err("Cannot open log file %s\n", filename);
 		return;
 	}
-	atexit(pr_closelog);
 }
 
 
@@ -83,13 +83,12 @@ void pr_openlog(const char *filename)
 int pr_msg(
 	FILE *fp,
 	const uint64_t flag,
-	const char *const fmt, ...)
+	const char *const fmt,
+	va_list ap)
 {
-	va_list ap;
 	int ret = 0;
 
-	va_start(ap, fmt);
-	if ((flag & PR_FAIL) || (opt_flags & flag)) {
+	if ((flag & PR_FAIL) || (g_opt_flags & flag)) {
 		char buf[4096];
 		const char *type = "";
 
@@ -102,47 +101,58 @@ int pr_msg(
 		if (flag & PR_FAIL)
 			type = "fail: ";
 
-		if (opt_flags & OPT_FLAGS_LOG_BRIEF) {
+		if (g_opt_flags & OPT_FLAGS_LOG_BRIEF) {
 			ret = vfprintf(fp, fmt, ap);
 		} else {
 			int n = snprintf(buf, sizeof(buf), "%s [%d] ",
 				type, (int)getpid());
 			ret = vsnprintf(buf + n, sizeof(buf) - n, fmt, ap);
-			fprintf(fp, "%s: %s", app_name, buf);
+			(void)fprintf(fp, "%s: %s", g_app_name, buf);
 		}
-		fflush(fp);
+		(void)fflush(fp);
 
 		if (flag & PR_FAIL) {
 			abort_fails++;
 			if (abort_fails >= ABORT_FAILURES) {
 				if (!abort_msg_emitted) {
 					abort_msg_emitted = true;
-					opt_do_run = false;
-					pr_msg(fp, PR_INFO, "%d failures "
+					g_keep_stressing_flag = false;
+					(void)fprintf(fp, "info: %d failures "
 						"reached, aborting stress "
 						"process\n", ABORT_FAILURES);
-					fflush(fp);
+					(void)fflush(fp);
 				}
 			}
 		}
 
 		/* Log messages to log file if --log-file specified */
 		if (log_file) {
-			fprintf(log_file, "%s: %s", app_name, buf);
-			fflush(log_file);
+			(void)fprintf(log_file, "%s: %s", g_app_name, buf);
+			(void)fflush(log_file);
 		}
 
 		/* Log messages if syslog requested, don't log DEBUG */
-		if ((opt_flags & OPT_FLAGS_SYSLOG) &&
+		if ((g_opt_flags & OPT_FLAGS_SYSLOG) &&
 		    (!(flag & PR_DEBUG))) {
 			syslog(LOG_INFO, "%s", buf);
 		}
 	}
-	va_end(ap);
-
 	return ret;
 }
 
+/*
+ *  __pr_msg_fail()
+ *	wrapper helper for pr_msg_fail
+ */
+static inline void __pr_msg_fail(const uint64_t flag, char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	(void)pr_msg(stderr, flag, fmt, ap);
+	va_end(ap);
+}
+
 /*
  *  pr_msg_fail()
  *	print failure message with errno
@@ -153,7 +163,98 @@ void pr_msg_fail(
 	const char *what,
 	const int err)
 {
-	pr_msg(stderr, flag, "%s: %s failed, errno=%d (%s)\n",
+	__pr_msg_fail(flag, "%s: %s failed, errno=%d (%s)\n",
 		name, what, err, strerror(err));
 }
 
+/*
+ *  pr_dbg()
+ *	print debug messages
+ */
+void pr_dbg(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	(void)pr_msg(stderr, PR_DEBUG, fmt, ap);
+	va_end(ap);
+}
+
+/*
+ *  pr_inf()
+ *	print info messages
+ */
+void pr_inf(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	(void)pr_msg(stderr, PR_INFO, fmt, ap);
+	va_end(ap);
+}
+
+/*
+ *  pr_err()
+ *	print error messages
+ */
+void pr_err(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	(void)pr_msg(stderr, PR_ERROR, fmt, ap);
+	va_end(ap);
+}
+
+/*
+ *  pr_fail()
+ *	print failure messages
+ */
+void pr_fail(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	(void)pr_msg(stderr, PR_FAIL, fmt, ap);
+	va_end(ap);
+}
+
+/*
+ *  pr_tidy()
+ *	print tidy up error messages
+ */
+void pr_tidy(const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	(void)pr_msg(stderr, g_caught_sigint ? PR_INFO : PR_DEBUG, fmt, ap);
+	va_end(ap);
+}
+
+/*
+ *  pr_fail_err__()
+ *	helper for macro pr_fail_err to print error with errno
+ */
+void pr_fail_err__(const args_t *args, const char *msg)
+{
+	pr_msg_fail(PR_FAIL | PR_ERROR, args->name, msg, errno);
+}
+
+/*
+ *  pr_fail_errno__()
+ *	helper for macro pr_fail_errno to print error with a given errno
+ */
+void pr_fail_errno__(const args_t *args, const char *msg, int err)
+{
+	pr_msg_fail(PR_FAIL | PR_ERROR, args->name, msg, err);
+}
+
+/*
+ *  pr_fail_dbg__()
+ *	helper for macro pr_fail_dbg to print error
+ */
+void pr_fail_dbg__(const args_t *args, const char *msg)
+{
+	pr_msg_fail(PR_DEBUG, args->name, msg, errno);
+}
