@@ -25,6 +25,15 @@
 #include "stress-ng.h"
 
 #if defined(__linux__)
+
+#define OOM_SCORE_ADJ_MIN	"-1000"
+#define OOM_SCORE_ADJ_MAX	"1000"
+
+#define OOM_ADJ_NO_OOM		"-17"
+#define OOM_ADJ_MIN		"-16"
+#define OOM_ADJ_MAX		"15"
+
+
 /*
  *  set_oom_adjustment()
  *	attempt to stop oom killer
@@ -35,9 +44,13 @@ void set_oom_adjustment(const char *name, const bool killable)
 {
 	int fd;
 	bool high_priv;
+	bool make_killable = killable;
 
 	high_priv = (getuid() == 0) && (geteuid() == 0);
 
+	if (g_opt_flags & OPT_FLAGS_OOMABLE)
+		make_killable = true;
+
 	/*
 	 *  Try modern oom interface
 	 */
@@ -45,10 +58,10 @@ void set_oom_adjustment(const char *name, const bool killable)
 		char *str;
 		ssize_t n;
 
-		if (killable)
-			str = "1000";
+		if (make_killable)
+			str = OOM_SCORE_ADJ_MAX;
 		else
-			str = high_priv ? "-1000" : "0";
+			str = high_priv ? OOM_SCORE_ADJ_MIN : "0";
 
 redo_wr1:
 		n = write(fd, str, strlen(str));
@@ -68,10 +81,10 @@ redo_wr1:
 		char *str;
 		ssize_t n;
 
-		if (killable)
-			str = high_priv ? "-17" : "-16";
+		if (make_killable)
+			str = high_priv ? OOM_ADJ_NO_OOM : OOM_ADJ_MIN;
 		else
-			str = "15";
+			str = OOM_ADJ_MAX;
 
 redo_wr2:
 		n = write(fd, str, strlen(str));
