@@ -35,7 +35,7 @@ void check_value(
 	if ((val < 0) || (val > STRESS_PROCS_MAX)) {
 		(void)fprintf(stderr, "Number of %s workers must be between "
 			"0 and %d\n", msg, STRESS_PROCS_MAX);
-		exit(EXIT_FAILURE);
+		longjmp(g_error_env, 1);
 	}
 }
 
@@ -54,7 +54,7 @@ void check_range(
 		(void)fprintf(stderr, "Value %" PRId64 " is out of range for %s,"
 			" allowed: %" PRId64 " .. %" PRId64 "\n",
 			val, opt, lo, hi);
-		exit(EXIT_FAILURE);
+		longjmp(g_error_env, 1);
 	}
 }
 
@@ -77,7 +77,7 @@ void check_range_bytes(
 			opt,
 			stress_uint64_to_str(strlo, sizeof(strlo), lo),
 			stress_uint64_to_str(strhi, sizeof(strhi), hi));
-		exit(EXIT_FAILURE);
+		longjmp(g_error_env, 1);
 	}
 }
 
@@ -102,7 +102,7 @@ static void ensure_positive(const char *const str)
 				return;
 
 			(void)fprintf(stderr, "Invalid negative number %s\n", str);
-			exit(EXIT_FAILURE);
+			longjmp(g_error_env, 1);
 		}
 	}
 }
@@ -118,7 +118,7 @@ uint32_t get_uint32(const char *const str)
 	ensure_positive(str);
 	if (sscanf(str, "%12" SCNu32, &val) != 1) {
 		(void)fprintf(stderr, "Invalid number %s\n", str);
-		exit(EXIT_FAILURE);
+		longjmp(g_error_env, 1);
 	}
 	return val;
 }
@@ -133,7 +133,7 @@ int32_t get_int32(const char *const str)
 
 	if (sscanf(str, "%12" SCNd32, &val) != 1) {
 		(void)fprintf(stderr, "Invalid number %s\n", str);
-		exit(EXIT_FAILURE);
+		longjmp(g_error_env, 1);
 	}
 	return val;
 }
@@ -149,7 +149,7 @@ uint64_t get_uint64(const char *const str)
 	ensure_positive(str);
 	if (sscanf(str, "%" SCNu64, &val) != 1) {
 		(void)fprintf(stderr, "Invalid number %s\n", str);
-		exit(EXIT_FAILURE);
+		longjmp(g_error_env, 1);
 	}
 	return val;
 }
@@ -171,7 +171,7 @@ uint64_t get_uint64_scale(
 	val = get_uint64(str);
 	if (!len)  {
 		(void)fprintf(stderr, "Value %s is an invalid size\n", str);
-		exit(EXIT_FAILURE);
+		longjmp(g_error_env, 1);
 	}
 	len--;
 	ch = str[len];
@@ -209,6 +209,46 @@ uint64_t get_uint64_byte(const char *const str)
 	return get_uint64_scale(str, scales, "length");
 }
 
+/*
+ *  get_uint64_percent()
+ *	get a value by whole number or by percentage
+ */
+uint64_t get_uint64_percent(
+	const char *const str,
+	const uint32_t instances,
+	const uint64_t max,
+	const char *const errmsg)
+{
+	size_t len = strlen(str);
+
+	/* Convert to % over N instances */
+	if ((len > 1) && (str[len - 1] == '%')) {
+		double val;
+
+		/* Avoid division by zero */
+		if (max == 0) {
+			(void)fprintf(stderr, "%s\n", errmsg);
+			longjmp(g_error_env, 1);
+		}
+
+		/* Should NEVER happen */
+		if (instances < 1) {
+			(void)fprintf(stderr, "Invalid number of instances\n");
+			longjmp(g_error_env, 1);
+		}
+		if (sscanf(str, "%lf", &val) != 1) {
+			(void)fprintf(stderr, "Invalid percentage %s\n", str);
+			longjmp(g_error_env, 1);
+		}
+		if (val < 0.0) {
+			(void)fprintf(stderr, "Invalid percentage %s\n", str);
+			longjmp(g_error_env, 1);
+		}
+		return (uint64_t)((double)(max * val) / (100.0 * instances));
+        }
+	return get_uint64_byte(str);
+}
+
 /*
  *  get_uint64_byte_memory()
  *	get memory size from string. If it contains %
@@ -220,29 +260,10 @@ uint64_t get_uint64_byte_memory(
 	const char *const str,
 	const uint32_t instances)
 {
-	size_t len = strlen(str);
+	uint64_t phys_mem = stress_get_phys_mem_size();
 
-	/* Convert to % of memory over N instances */
-	if ((len > 1) && (str[len - 1] == '%')) {
-		uint64_t phys_mem = stress_get_phys_mem_size();
-		double val;
-
-		/* Should NEVER happen */
-		if (instances < 1) {
-			(void)fprintf(stderr, "Invalid number of instances\n");
-			exit(EXIT_FAILURE);
-		}
-		if (sscanf(str, "%lf", &val) != 1) {
-			(void)fprintf(stderr, "Invalid percentage %s\n", str);
-			exit(EXIT_FAILURE);
-		}
-		if (phys_mem == 0) {
-			(void)fprintf(stderr, "Cannot determine physical memory size\n");
-			exit(EXIT_FAILURE);
-		}
-		return (uint64_t)((double)(phys_mem * val) / (100.0 * instances));
-        }
-	return get_uint64_byte(str);
+	return get_uint64_percent(str, instances, phys_mem,
+		"Cannot determine physical memory size");
 }
 
 /*
@@ -256,29 +277,10 @@ uint64_t get_uint64_byte_filesystem(
 	const char *const str,
 	const uint32_t instances)
 {
-	size_t len = strlen(str);
+	uint64_t bytes = stress_get_filesystem_size();
 
-	/* Convert to % of available filesystem space over N instances */
-	if ((len > 1) && (str[len - 1] == '%')) {
-		uint64_t bytes = stress_get_filesystem_size();
-		double val;
-
-		/* Should NEVER happen */
-		if (instances < 1) {
-			(void)fprintf(stderr, "Invalid number of instances\n");
-			exit(EXIT_FAILURE);
-		}
-		if (sscanf(str, "%lf", &val) != 1) {
-			(void)fprintf(stderr, "Invalid percentage %s\n", str);
-			exit(EXIT_FAILURE);
-		}
-		if (bytes == 0) {
-			(void)fprintf(stderr, "Cannot determine available space on file system\n");
-			exit(EXIT_FAILURE);
-		}
-		return (uint64_t)((double)(bytes * val) / (100.0 * instances));
-        }
-	return get_uint64_byte(str);
+	return get_uint64_percent(str, instances, bytes,
+		"Cannot determine available space on file system");
 }
 
 /*
