@@ -261,11 +261,12 @@ static inline int sys_perf_event_open(
  *  perf_yaml_label()
  *	turns text into a yaml compatible lable.
  */
-static char *perf_yaml_label(char *dst, const char *src, size_t n)
+static char *perf_yaml_label(char *dst, const char *src, const size_t n)
 {
 	if (n) {
 		char *d = dst;
 		const char *s = src;
+		size_t i = n;
 
 		do {
 			if (*s == ' ')
@@ -275,13 +276,13 @@ static char *perf_yaml_label(char *dst, const char *src, size_t n)
 			else if (*s)
 				*d = *s;
 			else {
-				while (--n != 0)
+				while (--i != 0)
 					*d++ = 0;
 				break;
 			}
 			s++;
 			d++;
-		} while (--n != 0);
+		} while (--i != 0);
 	}
 	return dst;
 }
@@ -299,7 +300,7 @@ int perf_open(stress_perf_t *sp)
 	if (g_shared->perf.no_perf)
 		return -1;
 
-	memset(sp, 0, sizeof(stress_perf_t));
+	(void)memset(sp, 0, sizeof(stress_perf_t));
 	sp->perf_opened = 0;
 
 	for (i = 0; i < STRESS_PERF_MAX; i++) {
@@ -311,7 +312,7 @@ int perf_open(stress_perf_t *sp)
 		if (perf_info[i].config != UNRESOLVED) {
 			struct perf_event_attr attr;
 
-			memset(&attr, 0, sizeof(attr));
+			(void)memset(&attr, 0, sizeof(attr));
 			attr.type = perf_info[i].type;
 			attr.config = perf_info[i].config;
 			attr.disabled = 1;
@@ -423,7 +424,7 @@ int perf_close(stress_perf_t *sp)
 			continue;
 		}
 
-		memset(&data, 0, sizeof(data));
+		(void)memset(&data, 0, sizeof(data));
 		ret = read(fd, &data, sizeof(data));
 		if (ret != sizeof(data))
 			sp->perf_stat[i].counter = STRESS_PERF_INVALID;
@@ -569,21 +570,16 @@ const char *perf_stat_scale(const uint64_t counter, const double duration)
 	return buffer;
 }
 
-void perf_stat_dump(
-	FILE *yaml,
-	const stress_t stressors[],
-	const proc_info_t procs[STRESS_MAX],
-	const int32_t max_procs,
-	const double duration)
+void perf_stat_dump(FILE *yaml, proc_info_t *procs_head, const double duration)
 {
-	int32_t i;
 	bool no_perf_stats = true;
+	proc_info_t *pi;
 
 	setlocale(LC_ALL, "");
 
 	pr_yaml(yaml, "perfstats:\n");
 
-	for (i = 0; i < STRESS_MAX; i++) {
+	for (pi = procs_head; pi; pi = pi->next) {
 		int p;
 		uint64_t counter_totals[STRESS_PERF_MAX];
 		uint64_t total_cpu_cycles = 0;
@@ -593,18 +589,18 @@ void perf_stat_dump(
 		bool got_data = false;
 		char *munged;
 
-		memset(counter_totals, 0, sizeof(counter_totals));
+		(void)memset(counter_totals, 0, sizeof(counter_totals));
 
 		/* Sum totals across all instances of the stressor */
 		for (p = 0; p < STRESS_PERF_MAX; p++) {
-			int32_t j, n = (i * max_procs);
-			stress_perf_t *sp = &g_shared->stats[n].sp;
+			int32_t j;
+			stress_perf_t *sp = &pi->stats[0]->sp;
 
 			if (!perf_stat_succeeded(sp))
 				continue;
 
 			ids[p] = ~0;
-			for (j = 0; j < procs[i].started_procs; j++, n++) {
+			for (j = 0; j < pi->started_procs; j++) {
 				uint64_t counter;
 
 				if (perf_get_counter_by_index(sp, p,
@@ -628,7 +624,7 @@ void perf_stat_dump(
 		if (!got_data)
 			continue;
 
-		munged = munge_underscore(stressors[i].name);
+		munged = munge_underscore(pi->stressor->name);
 		pr_inf("%s:\n", munged);
 		pr_yaml(yaml, "    - stressor: %s\n", munged);
 		pr_yaml(yaml, "      duration: %f\n", duration);
