@@ -25,6 +25,10 @@
 #include "stress-ng.h"
 #include <pwd.h>
 
+#if defined(__linux__) && defined(__NR_futex)
+#include <linux/futex.h>
+#endif
+
 /*
  *  Various shim abstraction wrappers around systems calls and
  *  GCC helper functions that may not be supported by some
@@ -106,7 +110,7 @@ static int shim_emulate_fallocate(int fd, off_t offset, off_t len)
 	if (n == (off_t)-1)
 		return -1;
 
-	memset(buffer, 0, buf_sz);
+	(void)memset(buffer, 0, buf_sz);
 	n = len;
 
 	while (g_keep_stressing_flag && (n > 0)) {
@@ -218,7 +222,7 @@ int shim_getdents(
 }
 
 /*
- *  shim_getdent6s()
+ *  shim_getdent64()
  *	wrapper for getdents64(2) - get directory entries
  */
 int shim_getdents64(
@@ -279,17 +283,17 @@ void shim_clear_cache(char* begin, char *end)
  *	wrapper for Linux kcmp(2) - compre two processes to
  *	see if they share a kernel resource.
  */
-long shim_kcmp(int pid1, int pid2, int type, int fd1, int fd2)
+long shim_kcmp(pid_t pid1, pid_t pid2, int type, unsigned long idx1, unsigned long idx2)
 {
 #if defined(__linux__) && defined(__NR_kcmp)
 	errno = 0;
-	return syscall(__NR_kcmp, pid1, pid2, type, fd1, fd2);
+	return syscall(__NR_kcmp, pid1, pid2, type, idx1, idx2);
 #else
 	(void)pid1;
 	(void)pid2;
 	(void)type;
-	(void)fd1;
-	(void)fd2;
+	(void)idx1;
+	(void)idx2;
 
 	errno = ENOSYS;
 	return -1;
@@ -669,6 +673,13 @@ int shim_usleep(uint64_t usec)
  */
 char *shim_getlogin(void)
 {
+#if defined(BUILD_STATIC)
+	/*
+	 *  static builds can't use getpwuid because of
+	 *  dynamic linking issues. Ugh.
+	 */
+	return NULL;
+#else
 	static char pw_name[256];
 	struct passwd *pw;
 
@@ -680,6 +691,7 @@ char *shim_getlogin(void)
 	pw_name[sizeof(pw_name) - 1 ] = '\0';
 
 	return pw_name;
+#endif
 }
 
 /*
@@ -749,7 +761,7 @@ int shim_sysfs(int option, ...)
  */
 int shim_madvise(void *addr, size_t length, int advice)
 {
-#if !defined(__gnu_hurd__) && NEED_GLIBC(2,19,0)
+#if defined(HAVE_MADVISE)
 	return madvise(addr, length, advice);
 #elif (_POSIX_C_SOURCE >= 200112L)
 	int posix_advice;
@@ -817,3 +829,159 @@ int shim_mincore(void *addr, size_t length, unsigned char *vec)
 	return -1;
 #endif
 }
+
+ssize_t shim_statx(
+	int dfd,
+	const char *filename,
+	unsigned int flags,
+	unsigned int mask,
+	struct shim_statx *buffer)
+{
+#if defined(__linux__) && defined(__NR_statx)
+	return syscall(__NR_statx, dfd, filename, flags, mask, buffer);
+#else
+	(void)dfd;
+	(void)filename;
+	(void)flags;
+	(void)mask;
+	(void)buffer;
+
+	errno = ENOSYS;
+	return -1;
+#endif
+}
+
+/*
+ *  futex wake()
+ *	wake n waiters on futex
+ */
+#if defined(__linux__) && defined(__NR_futex)
+int shim_futex_wake(const void *futex, const int n)
+{
+	return syscall(__NR_futex, futex, FUTEX_WAKE, n, NULL, NULL, 0);
+}
+#else
+int shim_futex_wake(const void *futex, const int n)
+{
+	(void)futex;
+	(void)n;
+
+	errno = -ENOSYS;
+	return -1;
+}
+#endif
+
+/*
+ *  futex_wait()
+ *	wait on futex with a timeout
+ */
+#if defined(__linux__) && defined(__NR_futex)
+int shim_futex_wait(
+	const void *futex,
+	const int val,
+	const struct timespec *timeout)
+{
+	return syscall(__NR_futex, futex, FUTEX_WAIT, val, timeout, NULL, 0);
+}
+#else
+int shim_futex_wait(
+	const void *futex,
+	const int val,
+	const struct timespec *timeout)
+{
+	(void)futex;
+	(void)val;
+	(void)timeout;
+
+	errno = -ENOSYS;
+	return -1;
+}
+#endif
+
+/*
+ *  dup3()
+ *	linux special dup
+ */
+#if defined(__linux__) && defined(__NR_dup3)
+int shim_dup3(int oldfd, int newfd, int flags)
+{
+	return syscall(__NR_dup3, oldfd, newfd, flags);
+}
+#else
+int shim_dup3(int oldfd, int newfd, int flags)
+{
+	(void)oldfd;
+	(void)newfd;
+	(void)flags;
+	
+	errno = -ENOSYS;
+	return -1;
+}
+#endif
+
+
+int shim_sync_file_range(
+	int fd,
+	shim_off64_t offset,
+	shim_off64_t nbytes,
+	unsigned int flags)
+{
+#if defined(__linux__) && defined(__NR_sync_file_range)
+	return syscall(__NR_sync_file_range, fd, offset, nbytes, flags);
+#elif defined(__linux__) && defined(__NR_sync_file_range2)
+	/*
+	 * from sync_file_range(2):
+	 * "Some architectures (e.g., PowerPC, ARM) need  64-bit  arguments
+	 * to be aligned in a suitable pair of registers.  On such
+	 * architectures, the call signature of sync_file_range() shown in
+	 * the SYNOPSIS would force a register to be wasted as padding
+	 * between the fd and offset arguments.  (See syscall(2) for details.)
+	 * Therefore, these architectures define a different system call that
+	 * orders the arguments suitably"
+	 */
+	return syscall(__NR_sync_file_range2, fd, flags, offset, nbytes);
+#else
+	(void)fd;
+	(void)offset;
+	(void)nbytes;
+	(void)flags;
+
+	errno = -ENOSYS;
+	return -1;
+#endif
+}
+
+/*
+ *  shim_ioprio_set()
+ *	ioprio_set system call
+ */
+int shim_ioprio_set(int which, int who, int ioprio)
+{
+#if defined(__linux__) && defined(__NR_ioprio_set)
+	return syscall(__NR_ioprio_set, which, who, ioprio);
+#else
+	(void)which;
+	(void)who;
+	(void)ioprio;
+
+	errno = ENOSYS;
+	return -1;
+#endif
+}
+
+/*
+ *  shim_ioprio_get()
+ *	ioprio_get system call
+ */
+int shim_ioprio_get(int which, int who)
+{
+#if defined(__linux__) && defined(__NR_ioprio_get)
+	return syscall(__NR_ioprio_get, which, who);
+#else
+	(void)which;
+	(void)who;
+
+	errno = ENOSYS;
+	return -1;
+#endif
+}
