@@ -35,20 +35,17 @@
 
 #endif
 
-#define BUFFER_SZ	(4096)
-
-static uint32_t opt_aio_linux_requests = DEFAULT_AIO_LINUX_REQUESTS;
-static bool set_aio_linux_requests = false;
+#define BUFFER_SZ		(4096)
+#define DEFAULT_AIO_MAX_NR	(65536)
 
 void stress_set_aio_linux_requests(const char *opt)
 {
-	uint32_t aio_linux_requests;
+	uint64_t aio_linux_requests;
 
-	set_aio_linux_requests = true;
 	aio_linux_requests = get_uint32(opt);
 	check_range("aiol-requests", aio_linux_requests,
 		MIN_AIO_LINUX_REQUESTS, MAX_AIO_LINUX_REQUESTS);
-	opt_aio_linux_requests = aio_linux_requests;
+	set_setting("aiol-requests", TYPE_ID_UINT64, &aio_linux_requests);
 }
 
 #if defined(__linux__) &&	\
@@ -81,20 +78,55 @@ int stress_aiol(const args_t *args)
 {
 	int fd, ret, rc = EXIT_FAILURE;
 	char filename[PATH_MAX];
+	char buf[64];
 	io_context_t ctx = 0;
+	uint64_t aio_linux_requests = DEFAULT_AIO_LINUX_REQUESTS;
+	uint8_t *buffer;
+	uint64_t aio_max_nr = DEFAULT_AIO_MAX_NR;
 
-	if (!set_aio_linux_requests) {
+	if (!get_setting("aiol-requests", &aio_linux_requests)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_aio_linux_requests = MAX_AIO_REQUESTS;
+			aio_linux_requests = MAX_AIO_REQUESTS;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_aio_linux_requests = MIN_AIO_REQUESTS;
+			aio_linux_requests = MIN_AIO_REQUESTS;
 	}
-	if ((opt_aio_linux_requests < MIN_AIO_REQUESTS) ||
-	    (opt_aio_linux_requests > MAX_AIO_REQUESTS)) {
+	if ((aio_linux_requests < MIN_AIO_REQUESTS) ||
+	    (aio_linux_requests > MAX_AIO_REQUESTS)) {
 		pr_err("%s: iol_requests out of range", args->name);
 		return EXIT_FAILURE;
 	}
-	ret = io_setup(opt_aio_linux_requests, &ctx);
+
+	ret = system_read("/proc/sys/fs/aio-max-nr", buf, sizeof(buf));
+	if (ret > 0) {
+		if (sscanf(buf, "%" SCNu64, &aio_max_nr) != 1) {
+			/* Guess max */
+			aio_max_nr = DEFAULT_AIO_MAX_NR;
+		}
+	} else {
+		/* Guess max */
+		aio_max_nr = DEFAULT_AIO_MAX_NR;
+	}
+
+	aio_max_nr /= (args->num_instances == 0) ? 1 : args->num_instances;
+
+	if (aio_max_nr < 1)
+		aio_max_nr = 1;
+	if (aio_linux_requests > aio_max_nr) {
+		aio_linux_requests = aio_max_nr;
+		if (args->instance == 0)
+			pr_inf("%s: Limiting AIO requests to "
+				"%" PRIu64 " per stressor (avoids running out of resources)\n",
+				args->name, aio_linux_requests);
+	}
+
+	ret = posix_memalign((void **)&buffer, 4096,
+		aio_linux_requests * BUFFER_SZ);
+	if (ret) {
+		pr_inf("%s: Out of memory allocating buffers, errno=%d (%s)",
+			args->name, errno, strerror(errno));
+		return EXIT_NO_RESOURCE;
+	}
+	ret = io_setup(aio_linux_requests, &ctx);
 	if (ret < 0) {
 		/*
 		 *  The libaio interface returns -errno in the
@@ -106,32 +138,38 @@ int stress_aiol(const args_t *args)
 				"available events, consider increasing "
 				"/proc/sys/fs/aio-max-nr, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			return EXIT_NO_RESOURCE;
+			rc = EXIT_NO_RESOURCE;
+			goto free_buffer;
 		} else if (errno == ENOMEM) {
 			pr_err("%s: io_setup failed, ran out of "
 				"memory, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			return EXIT_NO_RESOURCE;
+			rc = EXIT_NO_RESOURCE;
+			goto free_buffer;
 		} else if (errno == ENOSYS) {
 			pr_err("%s: io_setup failed, no io_setup "
 				"system call with this kernel, "
 				"errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			return EXIT_NO_RESOURCE;
+			rc = EXIT_NO_RESOURCE;
+			goto free_buffer;
 		} else {
 			pr_fail_err("io_setup");
-			return EXIT_FAILURE;
+			rc = EXIT_FAILURE;
+			goto free_buffer;
 		}
 	}
 	ret = stress_temp_dir_mk_args(args);
-	if (ret < 0)
-		return exit_status(-ret);
+	if (ret < 0) {
+		rc = exit_status(-ret);
+		goto free_buffer;
+	}
 
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), mwc32());
 
 	(void)umask(0077);
-	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
+	if ((fd = open(filename, O_CREAT | O_RDWR | O_DIRECT, S_IRUSR | S_IWUSR)) < 0) {
 		rc = exit_status(errno);
 		pr_fail_err("open");
 		goto finish;
@@ -139,18 +177,21 @@ int stress_aiol(const args_t *args)
 	(void)unlink(filename);
 
 	do {
-		struct iocb cb[opt_aio_linux_requests];
-		struct iocb *cbs[opt_aio_linux_requests];
-		struct io_event events[opt_aio_linux_requests];
-		uint8_t buffers[opt_aio_linux_requests][BUFFER_SZ];
-		uint32_t i;
+		struct iocb cb[aio_linux_requests];
+		struct iocb *cbs[aio_linux_requests];
+		struct io_event events[aio_linux_requests];
+		uint8_t *buffers[aio_linux_requests];
+		uint8_t *bufptr = buffer;
+		uint64_t i;
 		long n;
 
-		for (i = 0; i < opt_aio_linux_requests; i++)
+		for (i = 0; i < aio_linux_requests; i++, bufptr += BUFFER_SZ) {
+			buffers[i] = bufptr;
 			aio_linux_fill_buffer(i, buffers[i], BUFFER_SZ);
+		}
 
 		memset(cb, 0, sizeof(cb));
-		for (i = 0; i < opt_aio_linux_requests; i++) {
+		for (i = 0; i < aio_linux_requests; i++) {
 			cb[i].aio_fildes = fd;
 			cb[i].aio_lio_opcode = IO_CMD_PWRITE;
 			cb[i].u.c.buf = buffers[i];
@@ -158,7 +199,7 @@ int stress_aiol(const args_t *args)
 			cb[i].u.c.nbytes = BUFFER_SZ;
 			cbs[i] = &cb[i];
 		}
-		ret = io_submit(ctx, (long)opt_aio_linux_requests, cbs);
+		ret = io_submit(ctx, (long)aio_linux_requests, cbs);
 		if (ret < 0) {
 			errno = -ret;
 			if (errno == EAGAIN)
@@ -167,7 +208,7 @@ int stress_aiol(const args_t *args)
 			break;
 		}
 
-		n = opt_aio_linux_requests;
+		n = aio_linux_requests;
 		do {
 			struct timespec timeout, *timeout_ptr;
 
@@ -185,8 +226,12 @@ int stress_aiol(const args_t *args)
 			ret = io_getevents(ctx, 1, n, events, timeout_ptr);
 			if (ret < 0) {
 				errno = -ret;
-				if ((errno == EINTR) && (g_keep_stressing_flag))
-					continue;
+				if (errno == EINTR) {
+					if (g_keep_stressing_flag)
+						continue;
+					else
+						break;
+				}
 				pr_fail_err("io_getevents");
 				break;
 			} else {
@@ -201,6 +246,9 @@ int stress_aiol(const args_t *args)
 finish:
 	(void)io_destroy(ctx);
 	(void)stress_temp_dir_rm_args(args);
+
+free_buffer:
+	free(buffer);
 	return rc;
 }
 #else
