@@ -36,32 +36,19 @@ static int stress_cpu_online_set(
 	const int setting)
 {
 	char filename[PATH_MAX];
-	char data[3];
+	char data[3] = { '0' + setting, '\n', 0 };
 	ssize_t ret;
-	int fd, rc = EXIT_SUCCESS;
 
 	(void)snprintf(filename, sizeof(filename),
 		"/sys/devices/system/cpu/cpu%" PRId32 "/online", cpu);
-	fd = open(filename, O_WRONLY);
-	if (fd < 0) {
-		pr_fail_err("open");
+
+	ret = system_write(filename, data, sizeof data);
+	if ((ret < 0) &&
+	    ((ret != -EAGAIN) && (ret != -EINTR) && (ret != -EBUSY))) {
+		pr_fail_err("write");
 		return EXIT_FAILURE;
 	}
-
-	data[0] = '0' + setting;
-	data[1] = '\n';
-	data[2] = '\0';
-
-	ret = write(fd, data, 3);
-
-	if (ret != 3) {
-		if ((errno != EAGAIN) && (errno != EINTR)) {
-			pr_fail_err("write");
-			rc = EXIT_FAILURE;
-		}
-	}
-	(void)close(fd);
-	return rc;
+	return EXIT_SUCCESS;
 }
 
 /*
@@ -84,14 +71,15 @@ int stress_cpu_online(const args_t *args)
 	}
 
 	if ((cpus < 1) || (cpus > 65536)) {
-		pr_inf("%s: too few or too many CPUs (found %" PRId32 ")\n", args->name, cpus);
+		pr_inf("%s: too few or too many CPUs (found %" PRId32 ")\n",
+			args->name, cpus);
 		return EXIT_FAILURE;
 	}
 
 	cpu_online = calloc(cpus, sizeof(bool));
 	if (!cpu_online) {
 		pr_err("%s: out of memory\n", args->name);
-		return EXIT_FAILURE;
+		return EXIT_NO_RESOURCE;
 	}
 
 	/*
@@ -141,7 +129,7 @@ int stress_cpu_online(const args_t *args)
 	}
 	free(cpu_online);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 #else
 int stress_cpu_online(const args_t *args)
