@@ -44,6 +44,7 @@
 #define	csinl	csin
 #define cabsl	cabs
 #define sqrtl	sqrt
+#define lgammal	lgamma
 #endif
 
 #if defined(__FreeBSD__)
@@ -55,6 +56,7 @@
 
 #if defined(__minix__)
 #define cabsl	cabs
+#define lgammal	lgamma
 #endif
 
 /*
@@ -64,38 +66,38 @@ typedef void (*stress_cpu_func)(const char *name);
 
 typedef struct {
 	const char		*name;	/* human readable form of stressor */
-	const stress_cpu_func	func;	/* the stressor function */
-} stress_cpu_stressor_info_t;
+	const stress_cpu_func	func;	/* the cpu method function */
+} stress_cpu_method_info_t;
 
-/*
- *  opt_cpu_load_slice:
- *	< 0   - number of iterations per busy slice
- *	= 0   - random duration between 0..0.5 seconds
- *	> 0   - milliseconds per busy slice
- */
-static int32_t opt_cpu_load_slice = -64;
-static int32_t opt_cpu_load = 100;
-static const stress_cpu_stressor_info_t *opt_cpu_stressor;
-static const stress_cpu_stressor_info_t cpu_methods[];
+static const stress_cpu_method_info_t cpu_methods[];
 
 /* Don't make this static to ensure dithering does not get optimised out */
 uint8_t pixels[STRESS_CPU_DITHER_X][STRESS_CPU_DITHER_Y];
 
 void stress_set_cpu_load(const char *opt) {
-	opt_cpu_load = get_int32(opt);
-	if ((opt_cpu_load < 0) || (opt_cpu_load > 100)) {
-		(void)fprintf(stderr, "CPU load must in the range 0 to 100.\n");
-		exit(EXIT_FAILURE);
-	}
+	int32_t cpu_load;
+
+	cpu_load = get_int32(opt);
+	check_range("cpu-load", cpu_load, 0, 100);
+	set_setting("cpu-load", TYPE_ID_INT32, &cpu_load);
 }
 
+/*
+ *  stress_set_cpu_load_slice()
+ *	< 0   - number of iterations per busy slice
+ *	= 0   - random duration between 0..0.5 seconds
+ *	> 0   - milliseconds per busy slice
+ */
 void stress_set_cpu_load_slice(const char *opt)
 {
-	opt_cpu_load_slice = get_int32(opt);
-	if ((opt_cpu_load < -5000) || (opt_cpu_load > 5000)) {
-		(void)fprintf(stderr, "CPU load must in the range -5000 to 5000.\n");
+	int32_t cpu_load_slice;
+
+	cpu_load_slice = get_int32(opt);
+	if ((cpu_load_slice < -5000) || (cpu_load_slice > 5000)) {
+		(void)fprintf(stderr, "cpu-load-slice must in the range -5000 to 5000.\n");
 		exit(EXIT_FAILURE);
 	}
+	set_setting("cpu-load-slice", TYPE_ID_INT32, &cpu_load_slice);
 }
 
 /*
@@ -167,7 +169,7 @@ static void HOT OPTIMIZE3 stress_cpu_gcd(const char *name)
 	}
 	if ((g_opt_flags & OPT_FLAGS_VERIFY) && (i_sum != sum))
 		pr_fail("%s: gcd error detected, failed modulo "
-			"or assigment operations\n", name);
+			"or assignment operations\n", name);
 }
 
 /*
@@ -454,7 +456,7 @@ static void HOT stress_cpu_fft(const char *name)
 	for (i = 0; i < FFT_SIZE; i++)
 		buf[i] = (double complex)(i % 63);
 
-	memcpy(tmp, buf, sizeof(double complex) * FFT_SIZE);
+	(void)memcpy(tmp, buf, sizeof(double complex) * FFT_SIZE);
 	fft_partial(buf, tmp, FFT_SIZE, 1);
 }
 
@@ -831,7 +833,7 @@ stress_cpu_int(uint8_t, 8, \
 	0x12, 0x1a,
 	C1, C2, C3)
 
-#define float_ops(_type, a, b, c, d, sin, cos)		\
+#define float_ops(_type, a, b, c, d, _sin, _cos)	\
 	do {						\
 		a = a + b;				\
 		b = a * c;				\
@@ -840,14 +842,14 @@ stress_cpu_int(uint8_t, 8, \
 		a = c / (_type)0.1923;			\
 		b = c + a;				\
 		c = b * (_type)3.12;			\
-		d = d + b + (_type)sin(a);		\
+		d = d + b + (_type)_sin(a);		\
 		a = (b + c) / c;			\
 		b = b * c;				\
 		c = c + (_type)1.0;			\
-		d = d - (_type)sin(c);			\
-		a = a * (_type)cos(b);			\
-		b = b + (_type)cos(c);			\
-		c = (_type)sin(a) / (_type)2.344;	\
+		d = d - (_type)_sin(c);			\
+		a = a * (_type)_cos(b);			\
+		b = b + (_type)_cos(c);			\
+		c = (_type)_sin(a + b) / (_type)2.344;	\
 		b = d - (_type)1.0;			\
 	} while (0)
 
@@ -951,7 +953,7 @@ stress_cpu_complex(complex long double, l, complex_long_double, csinl, ccosl)
 		flt_a = flt_a * (_ftype)_cos(flt_b);		\
 		flt_b = flt_b + (_ftype)_cos(flt_c);		\
 		int_b &= (_c3);					\
-		flt_c = (_ftype)_sin(flt_a) / (_ftype)2.344;	\
+		flt_c = (_ftype)_sin(flt_a + flt_b) / (_ftype)2.344;	\
 		flt_b = flt_d - (_ftype)1.0;			\
 	} while (0)
 
@@ -1265,7 +1267,7 @@ static void stress_cpu_jmp(const char *name)  __attribute__((optimize("-O0")));
 		v = a;			\
 	else				\
 		v = b;			\
-	uint64_put(next + i);
+	uint64_put(next + i);		\
 
 /*
  *   stress_cpu_jmp
@@ -1594,15 +1596,48 @@ static void stress_cpu_hanoi(const char *name)
  *  factorial()
  *	compute n!
  */
-static long double HOT OPTIMIZE3 factorial(int n)
+static inline long double HOT OPTIMIZE3 factorial(int n)
 {
-	long double f = 1;
+	static long double factorials[] = {
+		1.0,
+		1.0,
+		2.0,
+		6.0,
+		24.0,
+		120.0,
+		720.0,
+		5040.0,
+		40320.0,
+		362880.0,
+		3628800.0,
+		39916800.0,
+		479001600.0,
+		6227020800.0,
+		87178291200.0,
+		1307674368000.0,
+		20922789888000.0,
+		355687428096000.0,
+		6402373705728000.0,
+		121645100408832000.0,
+		2432902008176640000.0,
+		51090942171709440000.0,
+		1124000727777607680000.0,
+		25852016738884976640000.0,
+		620448401733239439360000.0,
+		15511210043330985984000000.0,
+		403291461126605635592388608.0,
+		10888869450418352161430700032.0,
+		304888344611713860511469666304.0,
+		8841761993739701954695181369344.0,
+		265252859812191058647452510846976.0,
+		8222838654177922818071027836256256.0,
+		263130836933693530178272890760200192.0
+	};
 
-	while (n > 0) {
-		f *= (long double)n;
-		n--;
-	}
-	return f;
+	if (n < (int)SIZEOF_ARRAY(factorials))
+		return factorials[n];
+
+	return roundl(expl(lgammal((long double)(n + 1))));
 }
 
 /*
@@ -2076,6 +2111,43 @@ static void stress_cpu_queens(const char *name)
 	}
 }
 
+/*
+ *  stress_cpu_factorial
+ *	find factorials from 1..150 using
+ *	Stirling's and Ramanujan's Approximations.
+ */
+static void stress_cpu_factorial(const char *name)
+{
+	int n;
+	double f = 1.0;
+	const double precision = 1.0e-6;
+	const double sqrt_pi = sqrtl(M_PI);
+
+	for (n = 1; n < 150; n++) {
+		double fact = roundl(expl(lgammal((double)(n + 1))));
+		double dn;
+
+		f *= (double)n;
+
+		/* Stirling */
+		if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
+		    ((f - fact) / fact > precision)) {
+			pr_fail("%s: Stirling's approximation of factorial(%d) out of range\n",
+                                name, n);
+		}
+
+		/* Ramanujan */
+		dn = (double)n;
+		fact = sqrt_pi * powl((dn / M_E), dn);
+		fact *= powl((((((((8 * dn) + 4)) * dn) + 1) * dn) + 1.0/30.0), (1.0/6.0));
+		if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
+		    ((f - fact) / fact > precision)) {
+			pr_fail("%s: Ramanujan's approximation of factorial(%d) out of range\n",
+                                name, n);
+		}
+	}
+}
+
 /*
  *  stress_cpu_all()
  *	iterate over all cpu stressors
@@ -2092,7 +2164,7 @@ static HOT OPTIMIZE3 void stress_cpu_all(const char *name)
 /*
  * Table of cpu stress methods
  */
-static const stress_cpu_stressor_info_t cpu_methods[] = {
+static const stress_cpu_method_info_t cpu_methods[] = {
 	{ "all",		stress_cpu_all },	/* Special "all test */
 
 	{ "ackermann",		stress_cpu_ackermann },
@@ -2116,6 +2188,7 @@ static const stress_cpu_stressor_info_t cpu_methods[] = {
 	{ "euler",		stress_cpu_euler },
 	{ "explog",		stress_cpu_explog },
 	{ "fft",		stress_cpu_fft },
+	{ "factorial",		stress_cpu_factorial },
 	{ "fibonacci",		stress_cpu_fibonacci },
 	{ "float",		stress_cpu_float },
 	{ "fnv1a",		stress_cpu_fnv1a },
@@ -2180,13 +2253,13 @@ static const stress_cpu_stressor_info_t cpu_methods[] = {
  *  stress_set_cpu_method()
  *	set the default cpu stress method
  */
-int HOT OPTIMIZE3 stress_set_cpu_method(const char *name)
+int stress_set_cpu_method(const char *name)
 {
-	stress_cpu_stressor_info_t const *info = cpu_methods;
+	stress_cpu_method_info_t const *info;
 
 	for (info = cpu_methods; info->func; info++) {
 		if (!strcmp(info->name, name)) {
-			opt_cpu_stressor = info;
+			set_setting("cpu-method", TYPE_ID_UINTPTR_T, &info);
 			return 0;
 		}
 	}
@@ -2204,15 +2277,26 @@ int HOT OPTIMIZE3 stress_set_cpu_method(const char *name)
  *  stress_cpu()
  *	stress CPU by doing floating point math ops
  */
-int stress_cpu(const args_t *args)
+int HOT OPTIMIZE3 stress_cpu(const args_t *args)
 {
 	double bias;
-	stress_cpu_func func = opt_cpu_stressor->func;
+	const stress_cpu_method_info_t *cpu_method = &cpu_methods[0];
+	stress_cpu_func func;
+	int32_t cpu_load = 100;
+	int32_t cpu_load_slice = -64;
+
+	(void)get_setting("cpu-load", &cpu_load);
+	(void)get_setting("cpu-load-slice", &cpu_load_slice);
+	(void)get_setting("cpu-method", &cpu_method);
+
+	func = cpu_method->func;
+
+	pr_dbg("%s using method '%s'\n", args->name, cpu_method->name);
 
 	/*
 	 * Normal use case, 100% load, simple spinning on CPU
 	 */
-	if (opt_cpu_load == 100) {
+	if (cpu_load == 100) {
 		do {
 			(void)func(args->name);
 			inc_counter(args);
@@ -2224,7 +2308,7 @@ int stress_cpu(const args_t *args)
 	 * It is unlikely, but somebody may request to do a zero
 	 * load stress test(!)
 	 */
-	if (opt_cpu_load == 0) {
+	if (cpu_load == 0) {
 		sleep((int)g_opt_timeout);
 		return EXIT_SUCCESS;
 	}
@@ -2241,18 +2325,18 @@ int stress_cpu(const args_t *args)
 		struct timeval tv;
 
 		t1 = time_now();
-		if (opt_cpu_load_slice < 0) {
+		if (cpu_load_slice < 0) {
 			/* < 0 specifies number of iterations to do per slice */
 			int j;
 
-			for (j = 0; j < -opt_cpu_load_slice; j++) {
+			for (j = 0; j < -cpu_load_slice; j++) {
 				(void)func(args->name);
 				if (!g_keep_stressing_flag)
 					break;
 				inc_counter(args);
 			}
 			t2 = time_now();
-		} else if (opt_cpu_load_slice == 0) {
+		} else if (cpu_load_slice == 0) {
 			/* == 0, random time slices */
 			double slice_end = t1 + (((double)mwc16()) / 131072.0);
 			do {
@@ -2265,7 +2349,7 @@ int stress_cpu(const args_t *args)
 		} else {
 			/* > 0, time slice in milliseconds */
 			double slice_end = t1 +
-				((double)opt_cpu_load_slice / 1000.0);
+				((double)cpu_load_slice / 1000.0);
 			do {
 				(void)func(args->name);
 				t2 = time_now();
@@ -2276,7 +2360,7 @@ int stress_cpu(const args_t *args)
 		}
 		t = t2 - t1;
 		/* Must not calculate this with zero % load */
-		delay = t * (((100.0 / (double) opt_cpu_load)) - 1.0);
+		delay = t * (((100.0 / (double)cpu_load)) - 1.0);
 		delay -= bias;
 
 		tv.tv_sec = delay;
