@@ -32,27 +32,21 @@
  *  stress_eventfd
  *	stress eventfd read/writes
  */
-int stress_eventfd(
-	uint64_t *const counter,
-	const uint32_t instance,
-	const uint64_t max_ops,
-	const char *name)
+int stress_eventfd(const args_t *args)
 {
 	pid_t pid;
 	int fd1, fd2, rc;
 
-	(void)instance;
-
 	fd1 = eventfd(0, 0);
 	if (fd1 < 0) {
 		rc = exit_status(errno);
-		pr_fail_dbg(name, "eventfd");
+		pr_fail_dbg("eventfd");
 		return rc;
 	}
 	fd2 = eventfd(0, 0);
 	if (fd2 < 0) {
 		rc = exit_status(errno);
-		pr_fail_dbg(name, "eventfd");
+		pr_fail_dbg("eventfd");
 		(void)close(fd1);
 		return rc;
 	}
@@ -60,33 +54,33 @@ int stress_eventfd(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (opt_do_run && (errno == EAGAIN))
+		if (g_keep_stressing_flag && (errno == EAGAIN))
 			goto again;
-		pr_fail_dbg(name, "fork");
+		pr_fail_dbg("fork");
 		(void)close(fd1);
 		(void)close(fd2);
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		(void)setpgid(0, pgrp);
+		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 
-		while (opt_do_run) {
+		while (g_keep_stressing_flag) {
 			uint64_t val;
 			ssize_t ret;
 
 			for (;;) {
-				if (!opt_do_run)
+				if (!g_keep_stressing_flag)
 					goto exit_child;
 				ret = read(fd1, &val, sizeof(val));
 				if (ret < 0) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
-					pr_fail_dbg(name, "child read");
+					pr_fail_dbg("child read");
 					goto exit_child;
 				}
 				if (ret < (ssize_t)sizeof(val)) {
-					pr_fail_dbg(name, "child short read");
+					pr_fail_dbg("child short read");
 					goto exit_child;
 				}
 				break;
@@ -94,18 +88,18 @@ again:
 			val = 1;
 
 			for (;;) {
-				if (!opt_do_run)
+				if (!g_keep_stressing_flag)
 					goto exit_child;
 				ret = write(fd2, &val, sizeof(val));
 				if (ret < 0) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
-					pr_fail_dbg(name, "child write");
+					pr_fail_dbg("child write");
 					goto exit_child;
 				}
 				if (ret  < (ssize_t)sizeof(val)) {
-					pr_fail_dbg(name, "child short write");
+					pr_fail_dbg("child short write");
 					goto exit_child;
 				}
 				break;
@@ -123,7 +117,7 @@ exit_child:
 			int ret;
 
 			for (;;) {
-				if (!opt_do_run)
+				if (!g_keep_stressing_flag)
 					goto exit_parent;
 
 				ret = write(fd1, &val, sizeof(val));
@@ -131,18 +125,18 @@ exit_child:
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
-					pr_fail_dbg(name, "parent write");
+					pr_fail_dbg("parent write");
 					goto exit_parent;
 				}
 				if (ret < (ssize_t)sizeof(val)) {
-					pr_fail_dbg(name, "parent short write");
+					pr_fail_dbg("parent short write");
 					goto exit_parent;
 				}
 				break;
 			}
 
 			for (;;) {
-				if (!opt_do_run)
+				if (!g_keep_stressing_flag)
 					goto exit_parent;
 
 				ret = read(fd2, &val, sizeof(val));
@@ -150,17 +144,17 @@ exit_child:
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
-					pr_fail_dbg(name, "parent read");
+					pr_fail_dbg("parent read");
 					goto exit_parent;
 				}
 				if (ret < (ssize_t)sizeof(val)) {
-					pr_fail_dbg(name, "parent short read");
+					pr_fail_dbg("parent short read");
 					goto exit_parent;
 				}
 				break;
 			}
-			(*counter)++;
-		} while (opt_do_run && (!max_ops || *counter < max_ops));
+			inc_counter(args);
+		} while (keep_stressing());
 exit_parent:
 		(void)kill(pid, SIGKILL);
 		(void)waitpid(pid, &status, 0);
@@ -170,12 +164,8 @@ exit_parent:
 	return EXIT_SUCCESS;
 }
 #else
-int stress_eventfd(
-	uint64_t *const counter,
-	const uint32_t instance,
-	const uint64_t max_ops,
-	const char *name)
+int stress_eventfd(const args_t *args)
 {
-	return stress_not_implemented(counter, instance, max_ops, name);
+	return stress_not_implemented(args);
 }
 #endif
