@@ -23,7 +23,7 @@
  */
 #include "stress-ng.h"
 
-#if defined(__linux__)
+#if defined(HAVE_FANOTIFY)
 
 #include <mntent.h>
 #include <sys/select.h>
@@ -56,7 +56,7 @@ int stress_fanotify_supported(void)
 	return 0;
 }
 
-#if defined(__linux__)
+#if defined(HAVE_FANOTIFY)
 /*
  *  fanotify_event_init()
  *	initialize fanotify
