@@ -36,6 +36,8 @@ static inline bool fd_available(const int fd)
 	return ((ret == -1) && (errno == EBADF));
 }
 
+#if defined(F_DUPFD) || \
+    defined(F_DUPFD_CLOEXEC)
 /*
  *  fd_get()
  *	find a free fd to dup onto
@@ -54,6 +56,29 @@ static int fd_get(void)
 }
 #endif
 
+#endif
+
+#if defined(F_DUPFD) || 	\
+    defined(F_DUPFD_CLOEXEC) || \
+    defined(F_GETFD) ||		\
+    defined(F_SETFD) ||		\
+    defined(F_GETFL) ||		\
+    defined(F_SETFL) ||		\
+    defined(F_GETOWN) ||	\
+    defined(F_SETOWN) ||	\
+    defined(F_GETOWN_EX) ||	\
+    defined(F_SETOWN_EX) ||	\
+    defined(F_GETSIG) ||	\
+    defined(F_SETSIG) ||	\
+    defined(F_GETOWNER_UIDS) ||	\
+    defined(F_GETLEASE) ||	\
+    (defined(F_GETLK) && defined(F_SETLK) && \
+     defined(F_SETLKW) && defined(F_WRLCK) && \
+     defined(F_UNLCK)) ||	\
+    (defined(F_OFD_GETLK) && defined(F_OFD_SETLK) && \
+     defined(F_OFD_SETLKW) && defined(F_WRLCK) && defined(F_UNLCK)) | \
+    (defined(F_GET_FILE_RW_HINT) && defined(F_SET_FILE_RW_HINT))
+
 /*
  *  check_return()
  *	sanity check fcntl() return for errors
@@ -67,6 +92,9 @@ static void check_return(const args_t *args, const int ret, const char *cmd)
 	}
 }
 
+#endif
+
+
 /*
  *  do_fcntl()
  */
@@ -162,22 +190,29 @@ static int do_fcntl(const args_t *args, const int fd)
 	}
 #endif
 
-#if defined(F_SETOWN_EX)
+#if defined(F_SETOWN_EX) &&	\
+    (defined(F_OWNER_PID) ||	\
+     defined(F_OWNER_PGRP) ||	\
+     (defined(F_OWNER_TID) && defined(__linux__)))
 	{
 		int ret;
 		struct f_owner_ex owner;
 
+#if defined(F_OWNER_PID)
 		owner.type = F_OWNER_PID;
 		owner.pid = args->pid;
 		ret = fcntl(fd, F_SETOWN_EX, &owner);
 		check_return(args, ret, "F_SETOWN_EX, F_OWNER_PID");
+#endif
 
+#if defined(F_OWNER_PGRP)
 		owner.type = F_OWNER_PGRP;
 		owner.pid = getpgrp();
 		ret = fcntl(fd, F_SETOWN_EX, &owner);
 		check_return(args, ret, "F_SETOWN_EX, F_OWNER_PGRP");
+#endif
 
-#if defined(__linux__)
+#if defined(F_OWNER_TID) && defined(__linux__)
 		owner.type = F_OWNER_TID;
 		owner.pid = shim_gettid();
 		ret = fcntl(fd, F_SETOWN_EX, &owner);
@@ -402,6 +437,42 @@ ofd_lock_abort:	{ /* Nowt */ }
 	}
 #endif
 
+#if defined(F_GET_FILE_RW_HINT) && defined(F_SET_FILE_RW_HINT)
+	{
+		size_t i;
+		unsigned long hint;
+		static const unsigned long hints[] = {
+#if defined(RWH_WRITE_LIFE_EXTREME)
+			RWH_WRITE_LIFE_EXTREME,
+#endif
+#if defined(RWH_WRITE_LIFE_LONG)
+			RWH_WRITE_LIFE_LONG,
+#endif
+#if defined(RWH_WRITE_LIFE_MEDIUM)
+			RWH_WRITE_LIFE_MEDIUM,
+#endif
+#if defined(RWH_WRITE_LIFE_SHORT)
+			RWH_WRITE_LIFE_SHORT,
+#endif
+#if defined(RWH_WRITE_LIFE_NONE)
+			RWH_WRITE_LIFE_NONE,
+#endif
+#if defined(RWF_WRITE_LIFE_NOT_SET)
+			RWF_WRITE_LIFE_NOT_SET
+#endif
+		};
+
+		ret = fcntl(fd, F_GET_FILE_RW_HINT, &hint)
+		if (ret == 0) {
+			for (i = 0; i < SIZEOF_ARRAY(hints); i++) {
+				hint = hints[i];
+				ret = fcntl(fd, F_SET_FILE_RW_HINT, &hint);
+				(void)ret;
+			}
+		}
+	}
+#endif
+
 	return 0;
 }
 
