@@ -31,19 +31,18 @@ static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
 #endif
 
-static uint64_t opt_heapsort_size = DEFAULT_HEAPSORT_SIZE;
-static bool set_heapsort_size = false;
-
 /*
  *  stress_set_heapsort_size()
  *	set heapsort size
  */
 void stress_set_heapsort_size(const void *opt)
 {
-	set_heapsort_size = true;
-	opt_heapsort_size = get_uint64_byte(opt);
-	check_range("heapsort-size", opt_heapsort_size,
+	uint64_t heapsort_size;
+
+	heapsort_size = get_uint64(opt);
+	check_range("heapsort-size", heapsort_size,
 		MIN_HEAPSORT_SIZE, MAX_HEAPSORT_SIZE);
+	set_setting("heapsort-size", TYPE_ID_UINT64, &heapsort_size);
 }
 
 #if defined(HAVE_LIB_BSD)
@@ -107,18 +106,19 @@ static int stress_heapsort_cmp_3(const void *p1, const void *p2)
  */
 int stress_heapsort(const args_t *args)
 {
+	uint64_t heapsort_size = DEFAULT_HEAPSORT_SIZE;
 	int32_t *data, *ptr;
 	size_t n, i;
 	struct sigaction old_action;
 	int ret;
 
-	if (!set_heapsort_size) {
+	if (!get_setting("heapsort-size", &heapsort_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_heapsort_size = MAX_HEAPSORT_SIZE;
+			heapsort_size = MAX_HEAPSORT_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_heapsort_size = MIN_HEAPSORT_SIZE;
+			heapsort_size = MIN_HEAPSORT_SIZE;
 	}
-	n = (size_t)opt_heapsort_size;
+	n = (size_t)heapsort_size;
 
 	if ((data = calloc(n, sizeof(int32_t))) == NULL) {
 		pr_fail_dbg("malloc");
@@ -145,14 +145,18 @@ int stress_heapsort(const args_t *args)
 
 	do {
 		/* Sort "random" data */
-		heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_1);
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
-			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr > *(ptr+1)) {
-					pr_fail("%s: sort error "
-						"detected, incorrect ordering "
-						"found\n", args->name);
-					break;
+		if (heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_1) < 0) {
+			pr_fail("%s: heapsort of random data failed: %d (%s)\n",
+				args->name, errno, strerror(errno));
+		} else {
+			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
+					if (*ptr > *(ptr+1)) {
+						pr_fail("%s: sort error "
+							"detected, incorrect ordering "
+							"found\n", args->name);
+						break;
+					}
 				}
 			}
 		}
@@ -160,31 +164,42 @@ int stress_heapsort(const args_t *args)
 			break;
 
 		/* Reverse sort */
-		heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_2);
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
-			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr < *(ptr+1)) {
-					pr_fail("%s: reverse sort "
-						"error detected, incorrect "
-						"ordering found\n", args->name);
-					break;
+		if (heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_2) < 0) {
+			pr_fail("%s: reversed heapsort of random data failed: %d (%s)\n",
+				args->name, errno, strerror(errno));
+		} else {
+			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
+					if (*ptr < *(ptr+1)) {
+						pr_fail("%s: reverse sort "
+							"error detected, incorrect "
+							"ordering found\n", args->name);
+						break;
+					}
 				}
 			}
 		}
 		if (!g_keep_stressing_flag)
 			break;
 		/* And re-order by byte compare */
-		heapsort(data, n * 4, sizeof(uint8_t), stress_heapsort_cmp_3);
+		if (heapsort(data, n * 4, sizeof(uint8_t), stress_heapsort_cmp_3) < 0) {
+			pr_fail("%s: heapsort failed: %d (%s)\n",
+				args->name, errno, strerror(errno));
+		}
 
 		/* Reverse sort this again */
-		heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_2);
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
-			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr < *(ptr+1)) {
-					pr_fail("%s: reverse sort "
-						"error detected, incorrect "
-						"ordering found\n", args->name);
-					break;
+		if (heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_2) < 0) {
+			pr_fail("%s: reversed heapsort of random data failed: %d (%s)\n",
+				args->name, errno, strerror(errno));
+		} else {
+			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
+					if (*ptr < *(ptr+1)) {
+						pr_fail("%s: reverse sort "
+							"error detected, incorrect "
+							"ordering found\n", args->name);
+						break;
+					}
 				}
 			}
 		}
