@@ -25,8 +25,6 @@
 #include "stress-ng.h"
 
 static volatile uint64_t itimer_counter = 0;
-static uint64_t opt_itimer_freq = DEFAULT_TIMER_FREQ;
-static bool set_itimer_freq = false;
 static double rate_us;
 static double start;
 
@@ -36,10 +34,12 @@ static double start;
  */
 void stress_set_itimer_freq(const char *opt)
 {
-	set_itimer_freq = true;
-	opt_itimer_freq = get_uint64(opt);
-	check_range("itimer-freq", opt_itimer_freq,
+	uint64_t itimer_freq;
+
+	itimer_freq = get_uint64(opt);
+	check_range("itimer-freq", itimer_freq,
 		MIN_TIMER_FREQ, MAX_TIMER_FREQ);
+	set_setting("itimer-freq", TYPE_ID_UINT64, &itimer_freq);
 }
 
 /*
@@ -58,8 +58,8 @@ static void stress_itimer_set(struct itimerval *timer)
 		rate = rate_us;
 	}
 
-	timer->it_value.tv_sec = (long long int)rate / 1000000;
-	timer->it_value.tv_usec = (long long int)rate % 1000000;
+	timer->it_value.tv_sec = (time_t)rate / 1000000;
+	timer->it_value.tv_usec = (suseconds_t)rate % 1000000;
 	if (timer->it_value.tv_sec == 0 &&
 	    timer->it_value.tv_usec < 1)
 		timer->it_value.tv_usec = 1;
@@ -96,7 +96,7 @@ static void stress_itimer_handler(int sig)
 cancel:
 	g_keep_stressing_flag = false;
 	/* Cancel timer if we detect no more runs */
-	memset(&timer, 0, sizeof(timer));
+	(void)memset(&timer, 0, sizeof(timer));
 	(void)setitimer(ITIMER_PROF, &timer, NULL);
 }
 
@@ -108,20 +108,21 @@ int stress_itimer(const args_t *args)
 {
 	struct itimerval timer;
 	sigset_t mask;
+	uint64_t itimer_freq = DEFAULT_TIMER_FREQ;
 
-	sigemptyset(&mask);
-	sigaddset(&mask, SIGINT);
-	sigprocmask(SIG_SETMASK, &mask, NULL);
+	(void)sigemptyset(&mask);
+	(void)sigaddset(&mask, SIGINT);
+	(void)sigprocmask(SIG_SETMASK, &mask, NULL);
 
 	start = time_now();
 
-	if (!set_itimer_freq) {
+	if (!get_setting("itimer-freq", &itimer_freq)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_itimer_freq = MAX_ITIMER_FREQ;
+			itimer_freq = MAX_ITIMER_FREQ;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_itimer_freq = MIN_ITIMER_FREQ;
+			itimer_freq = MIN_ITIMER_FREQ;
 	}
-	rate_us = opt_itimer_freq ? 1000000 / opt_itimer_freq : 1000000;
+	rate_us = itimer_freq ? 1000000.0 / itimer_freq : 1000000.0;
 
 	if (stress_sighandler(args->name, SIGPROF, stress_itimer_handler, NULL) < 0)
 		return EXIT_FAILURE;
@@ -134,12 +135,12 @@ int stress_itimer(const args_t *args)
 
 	do {
 		struct itimerval t;
-		getitimer(ITIMER_PROF, &t);
+		(void)getitimer(ITIMER_PROF, &t);
 
 		*args->counter = itimer_counter;
 	} while (keep_stressing());
 
-	memset(&timer, 0, sizeof(timer));
+	(void)memset(&timer, 0, sizeof(timer));
 	(void)setitimer(ITIMER_PROF, &timer, NULL);
 	return EXIT_SUCCESS;
 }
