@@ -31,19 +31,18 @@ static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
 #endif
 
-static uint64_t opt_mergesort_size = DEFAULT_MERGESORT_SIZE;
-static bool set_mergesort_size = false;
-
 /*
  *  stress_set_mergesort_size()
  *	set mergesort size
  */
 void stress_set_mergesort_size(const void *opt)
 {
-	set_mergesort_size = true;
-	opt_mergesort_size = get_uint64_byte(opt);
-	check_range("mergesort-size", opt_mergesort_size,
+	uint64_t mergesort_size;
+
+	mergesort_size = get_uint64(opt);
+	check_range("mergesort-size", mergesort_size,
 		MIN_MERGESORT_SIZE, MAX_MERGESORT_SIZE);
+	set_setting("mergesort-size", TYPE_ID_UINT64, &mergesort_size);
 }
 
 #if defined(HAVE_LIB_BSD)
@@ -80,7 +79,7 @@ static int stress_mergesort_cmp_1(const void *p1, const void *p2)
 }
 
 /*
- *  stress_mergesort_cmp_1()
+ *  stress_mergesort_cmp_2()
  *	mergesort comparison - reverse sort on int32 values
  */
 static int stress_mergesort_cmp_2(const void *p1, const void *p2)
@@ -89,16 +88,16 @@ static int stress_mergesort_cmp_2(const void *p1, const void *p2)
 }
 
 /*
- *  stress_mergesort_cmp_1()
- *	mergesort comparison - sort on int8 values
+ *  stress_mergesort_cmp_3()
+ *	mergesort comparison - random sort(!)
  */
 static int stress_mergesort_cmp_3(const void *p1, const void *p2)
 {
-	const int8_t *i1 = (const int8_t *)p1;
-	const int8_t *i2 = (const int8_t *)p2;
+	(void)p1;
+	(void)p2;
+	int r = ((int)mwc8() % 3) - 1;
 
-	/* Force re-ordering on 8 bit value */
-	return *i1 - *i2;
+	return r;
 }
 
 /*
@@ -107,18 +106,19 @@ static int stress_mergesort_cmp_3(const void *p1, const void *p2)
  */
 int stress_mergesort(const args_t *args)
 {
+	uint64_t mergesort_size = DEFAULT_MERGESORT_SIZE;
 	int32_t *data, *ptr;
 	size_t n, i;
 	struct sigaction old_action;
 	int ret;
 
-	if (!set_mergesort_size) {
+	if (!get_setting("mergesort-size", &mergesort_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_mergesort_size = MAX_MERGESORT_SIZE;
+			mergesort_size = MAX_MERGESORT_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_mergesort_size = MIN_MERGESORT_SIZE;
+			mergesort_size = MIN_MERGESORT_SIZE;
 	}
-	n = (size_t)opt_mergesort_size;
+	n = (size_t)mergesort_size;
 
 	if ((data = calloc(n, sizeof(int32_t))) == NULL) {
 		pr_fail_dbg("malloc");
@@ -145,14 +145,18 @@ int stress_mergesort(const args_t *args)
 
 	do {
 		/* Sort "random" data */
-		mergesort(data, n, sizeof(uint32_t), stress_mergesort_cmp_1);
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
-			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr > *(ptr+1)) {
-					pr_fail("%s: sort error "
-						"detected, incorrect ordering "
-						"found\n", args->name);
-					break;
+		if (mergesort(data, n, sizeof(uint32_t), stress_mergesort_cmp_1) < 0) {
+			pr_fail("%s: mergesort of random data failed: %d (%s)\n",
+				args->name, errno, strerror(errno));
+		} else {
+			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
+					if (*ptr > *(ptr+1)) {
+						pr_fail("%s: sort error "
+							"detected, incorrect ordering "
+							"found\n", args->name);
+						break;
+					}
 				}
 			}
 		}
@@ -160,24 +164,34 @@ int stress_mergesort(const args_t *args)
 			break;
 
 		/* Reverse sort */
-		mergesort(data, n, sizeof(uint32_t), stress_mergesort_cmp_2);
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
-			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr < *(ptr+1)) {
-					pr_fail("%s: reverse sort "
-						"error detected, incorrect "
-						"ordering found\n", args->name);
-					break;
+		if (mergesort(data, n, sizeof(uint32_t), stress_mergesort_cmp_2) < 0) {
+			pr_fail("%s: reversed mergesort of random data failed: %d (%s)\n",
+                                args->name, errno, strerror(errno));
+		} else {
+			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
+					if (*ptr < *(ptr+1)) {
+						pr_fail("%s: reverse sort "
+							"error detected, incorrect "
+							"ordering found\n", args->name);
+						break;
+					}
 				}
 			}
 		}
 		if (!g_keep_stressing_flag)
 			break;
-		/* And re-order by byte compare */
-		mergesort(data, n * 4, sizeof(uint8_t), stress_mergesort_cmp_3);
+		/* And re-order by random compare to remix the data */
+		if (mergesort(data, n, sizeof(uint32_t), stress_mergesort_cmp_3) < 0) {
+			pr_fail("%s: mergesort failed: %d (%s)\n",
+				args->name, errno, strerror(errno));
+		}
 
 		/* Reverse sort this again */
-		mergesort(data, n, sizeof(uint32_t), stress_mergesort_cmp_2);
+		if (mergesort(data, n, sizeof(uint32_t), stress_mergesort_cmp_2) < 0) {
+			pr_fail("%s: reversed mergesort of random data failed: %d (%s)\n",
+				args->name, errno, strerror(errno));
+		}
 		if (g_opt_flags & OPT_FLAGS_VERIFY) {
 			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
 				if (*ptr < *(ptr+1)) {
