@@ -26,9 +26,6 @@
 
 #define NO_MEM_RETRIES_MAX	(256)
 
-static size_t opt_mmap_bytes = DEFAULT_MMAP_BYTES;
-static bool set_mmap_bytes = false;
-
 /* Misc randomly chosen mmap flags */
 static const int mmap_flags[] = {
 #if defined(MAP_HUGE_2MB) && defined(MAP_HUGETLB)
@@ -58,12 +55,12 @@ static const int mmap_flags[] = {
 
 void stress_set_mmap_bytes(const char *opt)
 {
-	set_mmap_bytes = true;
-	opt_mmap_bytes = (size_t)
-		get_uint64_byte_memory(opt,
-			stressor_instances(STRESS_MMAP));
-	check_range_bytes("mmap-bytes", opt_mmap_bytes,
+	size_t mmap_bytes;
+
+	mmap_bytes = (size_t)get_uint64_byte_memory(opt, 1);
+	check_range_bytes("mmap-bytes", mmap_bytes,
 		MIN_MMAP_BYTES, MAX_MEM_LIMIT);
+	set_setting("mmap-bytes", TYPE_ID_SIZE_T, &mmap_bytes);
 }
 
 /*
@@ -98,7 +95,8 @@ static void stress_mmap_child(
 	const int fd,
 	int *flags,
 	const size_t sz,
-	const size_t pages4k)
+	const size_t pages4k,
+	const size_t mmap_bytes)
 {
 	const size_t page_size = args->page_size;
 	int no_mem_retries = 0;
@@ -134,13 +132,13 @@ static void stress_mmap_child(
 			continue;	/* Try again */
 		}
 		if (g_opt_flags & OPT_FLAGS_MMAP_FILE) {
-			memset(buf, 0xff, sz);
+			(void)memset(buf, 0xff, sz);
 			(void)shim_msync((void *)buf, sz, ms_flags);
 		}
 		(void)madvise_random(buf, sz);
-		(void)mincore_touch_pages(buf, opt_mmap_bytes);
+		(void)mincore_touch_pages(buf, mmap_bytes);
 		stress_mmap_mprotect(args->name, buf, sz);
-		memset(mapped, PAGE_MAPPED, sizeof(mapped));
+		(void)memset(mapped, PAGE_MAPPED, sizeof(mapped));
 		for (n = 0; n < pages4k; n++)
 			mappings[n] = buf + (n * page_size);
 
@@ -155,7 +153,7 @@ static void stress_mmap_child(
 		/*
 		 *  Step #1, unmap all pages in random order
 		 */
-		(void)mincore_touch_pages(buf, opt_mmap_bytes);
+		(void)mincore_touch_pages(buf, mmap_bytes);
 		for (n = pages4k; n; ) {
 			uint64_t j, i = mwc64() % pages4k;
 			for (j = 0; j < n; j++) {
@@ -205,7 +203,7 @@ static void stress_mmap_child(
 							pr_fail("%s: mmap'd region of %zu bytes does "
 								"not contain expected data\n", args->name, page_size);
 						if (g_opt_flags & OPT_FLAGS_MMAP_FILE) {
-							memset(mappings[page], n, page_size);
+							(void)memset(mappings[page], n, page_size);
 #if !defined(__gnu_hurd__) && !defined(__minix__)
 							(void)msync((void *)mappings[page], page_size, ms_flags);
 #endif
@@ -242,6 +240,7 @@ int stress_mmap(const args_t *args)
 {
 	const size_t page_size = args->page_size;
 	size_t sz, pages4k;
+	size_t mmap_bytes = DEFAULT_MMAP_BYTES;
 	pid_t pid;
 	int fd = -1, flags = MAP_PRIVATE | MAP_ANONYMOUS;
 	uint32_t ooms = 0, segvs = 0, buserrs = 0;
@@ -250,13 +249,18 @@ int stress_mmap(const args_t *args)
 #if defined(MAP_POPULATE)
 	flags |= MAP_POPULATE;
 #endif
-	if (!set_mmap_bytes) {
+	if (!get_setting("mmap-bytes", &mmap_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_mmap_bytes = MAX_MMAP_BYTES;
+			mmap_bytes = MAX_MMAP_BYTES;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_mmap_bytes = MIN_MMAP_BYTES;
+			mmap_bytes = MIN_MMAP_BYTES;
 	}
-	sz = opt_mmap_bytes & ~(page_size - 1);
+	mmap_bytes /= args->num_instances;
+	if (mmap_bytes < MIN_MMAP_BYTES)
+		mmap_bytes = MIN_MMAP_BYTES;
+	if (mmap_bytes < page_size)
+		mmap_bytes = page_size;
+	sz = mmap_bytes & ~(page_size - 1);
 	pages4k = sz / page_size;
 
 	/* Make sure this is killable by OOM killer */
@@ -341,13 +345,22 @@ again:
 				args->instance);
 			/* If we got killed by OOM killer, re-start */
 			if (WTERMSIG(status) == SIGKILL) {
-				log_system_mem_info();
-				pr_dbg("%s: assuming killed by OOM "
-					"killer, restarting again "
-					"(instance %d)\n",
-					args->name, args->instance);
-				ooms++;
-				goto again;
+				if (g_opt_flags & OPT_FLAGS_OOMABLE) {
+					log_system_mem_info();
+					pr_dbg("%s: assuming killed by OOM "
+						"killer, bailing out "
+						"(instance %d)\n",
+						args->name, args->instance);
+					_exit(0);
+				} else {
+					log_system_mem_info();
+					pr_dbg("%s: assuming killed by OOM "
+						"killer, restarting again "
+						"(instance %d)\n",
+						args->name, args->instance);
+					ooms++;
+					goto again;
+				}
 			}
 			/* If we got killed by sigsegv, re-start */
 			if (WTERMSIG(status) == SIGSEGV) {
@@ -366,7 +379,7 @@ again:
 		/* Make sure this is killable by OOM killer */
 		set_oom_adjustment(args->name, true);
 
-		stress_mmap_child(args, fd, &flags, sz, pages4k);
+		stress_mmap_child(args, fd, &flags, sz, pages4k, mmap_bytes);
 	}
 
 cleanup:
