@@ -62,7 +62,7 @@ static void __strncat(char *dst, char *src, size_t *n)
 	if (*n <= ln)
 		return;
 
-	strncat(dst, src, *n);
+	(void)strncat(dst, src, *n);
 	*n -= ln;
 }
 
@@ -76,16 +76,12 @@ int stress_mmapfork(const args_t *args)
 	struct sysinfo info;
 	void *ptr;
 	uint64_t segv_count = 0;
-	int32_t instances;
 	int8_t segv_reasons = 0;
 
-	if ((instances = stressor_instances(STRESS_MMAPFORK)) < 1)
-		instances = stress_get_processors_configured();
-
 	do {
 		size_t i, n, len;
 
-		memset(pids, 0, sizeof(pids));
+		(void)memset(pids, 0, sizeof(pids));
 
 		for (n = 0; n < MAX_PIDS; n++) {
 retry:			if (!g_keep_stressing_flag)
@@ -113,7 +109,7 @@ retry:			if (!g_keep_stressing_flag)
 					_exit(_EXIT_FAILURE);
 				}
 
-				len = ((size_t)info.freeram / (instances * MAX_PIDS)) / 2;
+				len = ((size_t)info.freeram / (args->num_instances * MAX_PIDS)) / 2;
 				segv_ret = _EXIT_SEGV_MMAP;
 				ptr = mmap(NULL, len, PROT_READ | PROT_WRITE,
 					MAP_POPULATE | MAP_SHARED | MAP_ANONYMOUS, -1, 0);
@@ -122,13 +118,13 @@ retry:			if (!g_keep_stressing_flag)
 					(void)shim_madvise(ptr, len, MADV_WILLNEED);
 
 					segv_ret = _EXIT_SEGV_MEMSET;
-					memset(ptr, 0, len);
+					(void)memset(ptr, 0, len);
 
 					segv_ret = _EXIT_SEGV_MADV_DONTNEED;
 					(void)shim_madvise(ptr, len, MADV_DONTNEED);
 
 					segv_ret = _EXIT_SEGV_MUNMAP;
-					munmap(ptr, len);
+					(void)munmap(ptr, len);
 				}
 				_exit(EXIT_SUCCESS);
 			}
