@@ -99,12 +99,18 @@ static int monitor(const args_t *args, const int sock)
 		case PROC_EVENT_FORK:
 		case PROC_EVENT_EXEC:
 		case PROC_EVENT_EXIT:
+		case PROC_EVENT_UID:
+		case PROC_EVENT_SID:
+		
 #endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0)
 		case PROC_EVENT_COREDUMP:
 #endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,1,0)
 		case PROC_EVENT_COMM:
+#endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,1,0)
+		case PROC_EVENT_PTRACE:
 #endif
 			inc_counter(args);
 			break;
@@ -175,7 +181,7 @@ int stress_netlink_proc(const args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	memset(&addr, 0, sizeof(addr));
+	(void)memset(&addr, 0, sizeof(addr));
 	addr.nl_pid = getpid();
 	addr.nl_family = AF_NETLINK;
 	addr.nl_groups = CN_IDX_PROC;
@@ -187,14 +193,14 @@ int stress_netlink_proc(const args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	memset(&nlmsghdr, 0, sizeof(nlmsghdr));
+	(void)memset(&nlmsghdr, 0, sizeof(nlmsghdr));
 	nlmsghdr.nlmsg_len = NLMSG_LENGTH(sizeof(cn_msg) + sizeof(op));
 	nlmsghdr.nlmsg_pid = getpid();
 	nlmsghdr.nlmsg_type = NLMSG_DONE;
 	iov[0].iov_base = &nlmsghdr;
 	iov[0].iov_len = sizeof(nlmsghdr);
 
-	memset(&cn_msg, 0, sizeof(cn_msg));
+	(void)memset(&cn_msg, 0, sizeof(cn_msg));
 	cn_msg.id.idx = CN_IDX_PROC;
 	cn_msg.id.val = CN_VAL_PROC;
 	cn_msg.len = sizeof(enum proc_cn_mcast_op);
