@@ -65,13 +65,13 @@ int stress_nice(const args_t *args)
 			pid = getpid();
 			for (i = min_prio; i <= max_prio; i++) {
 				errno = 0;
-				setpriority(PRIO_PROCESS, pid, i);
+				(void)setpriority(PRIO_PROCESS, pid, i);
 				if (!errno) {
 					double start = time_now();
 					double delay = 0.05 + (double)mwc16() / 327680.0;
 
 					while (time_now() - start < delay)
-						;
+						shim_sched_yield();
 					inc_counter(args);
 				}
 			}
