@@ -89,13 +89,31 @@ int stress_ptrace(const args_t *args)
 		 *  A simple mix of system calls
 		 */
 		while (g_keep_stressing_flag) {
-			(void)getppid();
-			(void)getgid();
-			(void)getegid();
-			(void)getuid();
-			(void)geteuid();
-			(void)getpgrp();
-			(void)time(NULL);
+			pid_t pidtmp;
+			gid_t gidtmp;
+			uid_t uidtmp;
+			time_t ttmp;
+
+			pidtmp = getppid();
+			(void)pidtmp;
+
+			pidtmp = getpgrp();
+			(void)pidtmp;
+
+			gidtmp = getgid();
+			(void)gidtmp;
+
+			gidtmp = getegid();
+			(void)gidtmp;
+
+			uidtmp = getuid();
+			(void)uidtmp;
+
+			uidtmp = geteuid();
+			(void)uidtmp;
+
+			ttmp = time(NULL);
+			(void)ttmp;
 		}
 		_exit(0);
 	} else {
