@@ -37,6 +37,13 @@
 #if defined(HAVE_LIB_PTHREAD) && defined(__linux__)
 #include <semaphore.h>
 #endif
+#if defined(HAVE_MQ_SYSV)
+#include <sys/ipc.h>
+#include <sys/msg.h>
+#endif
+#if defined(HAVE_LIB_RT) && defined(HAVE_MQ_POSIX)
+#include <mqueue.h>
+#endif
 
 #define RESOURCE_FORKS 	(1024)
 #define MAX_LOOPS	(1024)
@@ -80,10 +87,20 @@ typedef struct {
 	bool timerok;
 	timer_t timerid;
 #endif
-#if defined(HAVE_LIB_PTHREAD) && defined(__linux__)
+#if defined(HAVE_LIB_PTHREAD) && (HAVE_SEM_POSIX)
 	bool semok;
 	sem_t sem;
 #endif
+#if defined(HAVE_SEM_SYSV)
+	int sem_id;
+#endif
+#if defined(HAVE_MQ_SYSV)
+	int msgq_id;
+#endif
+#if defined(HAVE_LIB_RT) && defined(HAVE_MQ_POSIX)
+	mqd_t mq;
+	char mq_name[64];
+#endif
 } info_t;
 
 static pid_t pids[RESOURCE_FORKS];
@@ -99,12 +116,13 @@ static void *stress_pthread_func(void *ctxt)
 	static void *nowt;
 
 	(void)ctxt;
-	sleep(1);
+	(void)sleep(1);
 	return &nowt;
 }
 #endif
 
 static void NORETURN waste_resources(
+	const args_t *args,
 	const size_t page_size,
 	const size_t pipe_size)
 {
@@ -112,12 +130,15 @@ static void NORETURN waste_resources(
 #if defined(__NR_memfd_create) || defined(O_TMPFILE)
 	const pid_t pid = getpid();
 #endif
-
 	static int domains[] = { AF_INET, AF_INET6 };
 	static int types[] = { SOCK_STREAM, SOCK_DGRAM };
 	info_t info[MAX_LOOPS];
 
-	memset(&info, 0, sizeof(info));
+#if !(defined(HAVE_LIB_RT) && defined(HAVE_MQ_POSIX))
+	(void)args;
+#endif
+
+	(void)memset(&info, 0, sizeof(info));
 
 	for (i = 0; g_keep_stressing_flag && (i < MAX_LOOPS); i++) {
 #if defined(__NR_memfd_create)
@@ -141,10 +162,11 @@ static void NORETURN waste_resources(
 		if (!(mwc32() & 0xf)) {
 			info[i].m_mmap_size = page_size;
 			info[i].m_mmap = mmap(NULL, info[i].m_mmap_size,
-				PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
+				PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 			if (!g_keep_stressing_flag)
 				break;
 			if (info[i].m_mmap != MAP_FAILED) {
+				(void)madvise_random(info[i].m_mmap, info[i].m_mmap_size);
 				mincore_touch_pages(info[i].m_mmap, info[i].m_mmap_size);
 				if (!g_keep_stressing_flag)
 					break;
@@ -259,9 +281,35 @@ static void NORETURN waste_resources(
 				(timer_create(CLOCK_REALTIME, &sevp, &info[i].timerid) == 0);
 		}
 #endif
+
 #if defined(HAVE_LIB_PTHREAD) && defined(__linux__)
 		info[i].semok = (sem_init(&info[i].sem, 1, 1) >= 0);
 #endif
+
+#if defined(HAVE_SEM_SYSV)
+		key_t sem_key = (key_t)mwc32();
+		info[i].sem_id = semget(sem_key, 1,
+			IPC_CREAT | S_IRUSR | S_IWUSR);
+#endif
+
+#if defined(HAVE_MQ_SYSV)
+		info[i].msgq_id = msgget(IPC_PRIVATE,
+				S_IRUSR | S_IWUSR | IPC_CREAT | IPC_EXCL);
+#endif
+
+#if defined(HAVE_LIB_RT) && defined(HAVE_MQ_POSIX)
+		struct mq_attr attr;
+
+		snprintf(info[i].mq_name, sizeof(info[i].mq_name), "/%s-%i-%" PRIu32 "-%zu",
+			args->name, getpid(), args->instance, i);
+		attr.mq_flags = 0;
+		attr.mq_maxmsg = 1;
+		attr.mq_msgsize = 32;
+		attr.mq_curmsgs = 0;
+
+		info[i].mq = mq_open(info[i].mq_name,
+			O_CREAT | O_RDWR, S_IRUSR | S_IWUSR, &attr);
+#endif
 	}
 
 	for (i = 0; g_keep_stressing_flag && (i < MAX_LOOPS); i++) {
@@ -324,10 +372,28 @@ static void NORETURN waste_resources(
 		if (info[i].pty_master != -1)
 			(void)close(info[i].pty_master);
 #endif
+
 #if defined(HAVE_LIB_PTHREAD) && defined(__linux__)
 		if (info[i].semok)
 			(void)sem_destroy(&info[i].sem);
 #endif
+
+#if defined(HAVE_SEM_SYSV)
+		if (info[i].sem_id >= 0)
+			(void)semctl(info[i].sem_id, 0, IPC_RMID);
+#endif
+
+#if defined(HAVE_MQ_SYSV)
+		if (info[i].msgq_id >= 0)
+			(void)msgctl(info[i].msgq_id, IPC_RMID, NULL);
+#endif
+
+#if defined(HAVE_LIB_RT) && defined(HAVE_MQ_POSIX)
+		if (info[i].mq >= 0) {
+			(void)mq_close(info[i].mq);
+			(void)mq_unlink(info[i].mq_name);
+		}
+#endif
 	}
 	_exit(0);
 }
@@ -379,7 +445,7 @@ int stress_resources(const args_t *args)
 	do {
 		unsigned int i;
 
-		memset(pids, 0, sizeof(pids));
+		(void)memset(pids, 0, sizeof(pids));
 		for (i = 0; i < RESOURCE_FORKS; i++) {
 			pid_t pid = fork();
 
@@ -389,7 +455,7 @@ int stress_resources(const args_t *args)
 				if (ret)
 					_exit(0);
 				set_oom_adjustment(args->name, true);
-				waste_resources(page_size, pipe_size);
+				waste_resources(args, page_size, pipe_size);
 				_exit(0); /* should never get here */
 			}
 			if (pid > -1)
