@@ -24,10 +24,7 @@
  */
 #include "stress-ng.h"
 
-#if defined(__linux__)
-#include <sys/ipc.h>
-#include <sys/sem.h>
-
+#if defined(HAVE_SEM_SYSV)
 typedef union _semun {
 	int              val;	/* Value for SETVAL */
 	struct semid_ds *buf;	/* Buffer for IPC_STAT, IPC_SET */
@@ -36,18 +33,17 @@ typedef union _semun {
 } semun_t;
 #endif
 
-static uint64_t opt_semaphore_sysv_procs = DEFAULT_SEMAPHORE_PROCS;
-static bool set_semaphore_sysv_procs = false;
-
 void stress_set_semaphore_sysv_procs(const char *opt)
 {
-	set_semaphore_sysv_procs = true;
-	opt_semaphore_sysv_procs = get_uint64_byte(opt);
-	check_range("sem-procs", opt_semaphore_sysv_procs,
+	uint64_t semaphore_sysv_procs;
+
+	semaphore_sysv_procs = get_uint64(opt);
+	check_range("sem-sysv-procs", semaphore_sysv_procs,
 		MIN_SEMAPHORE_PROCS, MAX_SEMAPHORE_PROCS);
+	set_setting("sem-sysv-procs", TYPE_ID_UINT64, &semaphore_sysv_procs);
 }
 
-#if defined(__linux__)
+#if defined(HAVE_SEM_SYSV)
 
 /*
  *  stress_semaphore_sysv_init()
@@ -130,7 +126,11 @@ static void semaphore_sysv_thrash(const args_t *args)
 			semsignal.sem_op = 1;
 			semsignal.sem_flg = SEM_UNDO;
 
+#if defined(__linux__)
 			if (semtimedop(sem_id, &semwait, 1, &timeout) < 0) {
+#else
+			if (semop(sem_id, &semwait, 1) < 0) {
+#endif
 				if (errno == EAGAIN) {
 					pr_inf("Semaphore timed out: errno=%d (%s)\n",
 						errno, strerror(errno));
@@ -170,7 +170,7 @@ timed_out:
 				pr_fail_dbg("semctl SEM_STAT");
 		}
 #endif
-#if defined(IPC_INFO)
+#if defined(IPC_INFO) && defined(__linux__)
 		{
 			struct seminfo si;
 			semun_t s;
@@ -180,7 +180,7 @@ timed_out:
 				pr_fail_dbg("semctl IPC_INFO");
 		}
 #endif
-#if defined(SEM_INFO)
+#if defined(SEM_INFO) && defined(__linux__)
 		{
 			struct seminfo si;
 			semun_t s;
@@ -243,12 +243,13 @@ int stress_sem_sysv(const args_t *args)
 {
 	pid_t pids[MAX_SEMAPHORE_PROCS];
 	uint64_t i;
+	uint64_t semaphore_sysv_procs = DEFAULT_SEMAPHORE_PROCS;
 
-	if (!set_semaphore_sysv_procs) {
+	if (!get_setting("sem-sysv-procs", &semaphore_sysv_procs)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_semaphore_sysv_procs = MAX_SEMAPHORE_PROCS;
+			semaphore_sysv_procs = MAX_SEMAPHORE_PROCS;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_semaphore_sysv_procs = MIN_SEMAPHORE_PROCS;
+			semaphore_sysv_procs = MIN_SEMAPHORE_PROCS;
 	}
 
 	if (!g_shared->sem_sysv.init) {
@@ -256,8 +257,8 @@ int stress_sem_sysv(const args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	memset(pids, 0, sizeof(pids));
-	for (i = 0; i < opt_semaphore_sysv_procs; i++) {
+	(void)memset(pids, 0, sizeof(pids));
+	for (i = 0; i < semaphore_sysv_procs; i++) {
 		pids[i] = semaphore_sysv_spawn(args);
 		if (!g_keep_stressing_flag || pids[i] < 0)
 			goto reap;
@@ -266,11 +267,11 @@ int stress_sem_sysv(const args_t *args)
 	while (keep_stressing())
 		(void)shim_usleep(100000);
 reap:
-	for (i = 0; i < opt_semaphore_sysv_procs; i++) {
+	for (i = 0; i < semaphore_sysv_procs; i++) {
 		if (pids[i] > 0)
 			(void)kill(pids[i], SIGKILL);
 	}
-	for (i = 0; i < opt_semaphore_sysv_procs; i++) {
+	for (i = 0; i < semaphore_sysv_procs; i++) {
 		if (pids[i] > 0) {
 			int status;
 
