@@ -27,31 +27,28 @@
 #define SHM_NAME_LEN	128
 
 typedef struct {
-	int	index;
+	ssize_t	index;
 	char	shm_name[SHM_NAME_LEN];
 } shm_msg_t;
 
-static size_t opt_shm_posix_bytes = DEFAULT_SHM_POSIX_BYTES;
-static size_t opt_shm_posix_objects = DEFAULT_SHM_POSIX_OBJECTS;
-static bool set_shm_posix_bytes = false;
-static bool set_shm_posix_objects = false;
-
 void stress_set_shm_posix_bytes(const char *opt)
 {
-	set_shm_posix_bytes = true;
-	opt_shm_posix_bytes = (size_t)
-		get_uint64_byte_memory(opt,
-			stressor_instances(STRESS_SHM_POSIX));
-	check_range_bytes("shm-bytes", opt_shm_posix_bytes,
+	size_t shm_posix_bytes;
+
+	shm_posix_bytes = (size_t)get_uint64_byte_memory(opt, 1);
+	check_range_bytes("shm-bytes", shm_posix_bytes,
 		MIN_SHM_POSIX_BYTES, MAX_MEM_LIMIT);
+	set_setting("shm-bytes", TYPE_ID_SIZE_T, &shm_posix_bytes);
 }
 
 void stress_set_shm_posix_objects(const char *opt)
 {
-	set_shm_posix_objects = true;
-	opt_shm_posix_objects = (size_t)get_uint64_byte(opt);
-	check_range("shm-segments", opt_shm_posix_objects,
+	size_t shm_posix_objects;
+
+	shm_posix_objects = (size_t)get_uint64(opt);
+	check_range("shm-objs", shm_posix_objects,
 		MIN_SHM_POSIX_OBJECTS, MAX_48);
+	set_setting("shm-objs", TYPE_ID_SIZE_T, &shm_posix_objects);
 }
 
 #if defined(HAVE_LIB_RT)
@@ -68,7 +65,7 @@ static int stress_shm_posix_check(
 	uint8_t *ptr, *end = buf + sz;
 	uint8_t val;
 
-	memset(buf, 0xa5, sz);
+	(void)memset(buf, 0xa5, sz);
 
 	for (val = 0, ptr = buf; ptr < end; ptr += page_size, val++) {
 		*ptr = val;
@@ -92,10 +89,11 @@ static int stress_shm_posix_check(
 static int stress_shm_posix_child(
 	const args_t *args,
 	const int fd,
-	size_t sz)
+	size_t sz,
+	size_t shm_posix_objects)
 {
-	void *addrs[MAX_SHM_POSIX_OBJECTS];
-	char shm_names[MAX_SHM_POSIX_OBJECTS][SHM_NAME_LEN];
+	void **addrs;
+	char *shm_names;
 	shm_msg_t msg;
 	int i;
 	int rc = EXIT_SUCCESS;
@@ -104,17 +102,26 @@ static int stress_shm_posix_child(
 	uint64_t id = 0;
 	const size_t page_size = args->page_size;
 
-	memset(addrs, 0, sizeof(addrs));
-	memset(shm_names, 0, sizeof(shm_names));
+	addrs = calloc(shm_posix_objects, sizeof(void *));
+	if (!addrs) {
+		pr_fail_err("calloc on addrs");
+		return EXIT_NO_RESOURCE;
+	}
+	shm_names = calloc(shm_posix_objects, SHM_NAME_LEN);
+	if (!shm_names) {
+		free(addrs);
+		pr_fail_err("calloc on shm_names");
+		return EXIT_NO_RESOURCE;
+	}
 
 	/* Make sure this is killable by OOM killer */
 	set_oom_adjustment(args->name, true);
 
 	do {
-		for (i = 0; ok && (i < (ssize_t)opt_shm_posix_objects); i++) {
+		for (i = 0; ok && (i < (ssize_t)shm_posix_objects); i++) {
 			int shm_fd;
 			void *addr;
-			char *shm_name = shm_names[i];
+			char *shm_name = &shm_names[i * SHM_NAME_LEN];
 
 			shm_name[0] = '\0';
 
@@ -137,7 +144,7 @@ static int stress_shm_posix_child(
 			/* Inform parent of the new shm name */
 			msg.index = i;
 			shm_name[SHM_NAME_LEN - 1] = '\0';
-			strncpy(msg.shm_name, shm_name, SHM_NAME_LEN);
+			(void)strncpy(msg.shm_name, shm_name, SHM_NAME_LEN);
 			if (write(fd, &msg, sizeof(msg)) < 0) {
 				pr_err("%s: write failed: errno=%d: (%s)\n",
 					args->name, errno, strerror(errno));
@@ -147,7 +154,7 @@ static int stress_shm_posix_child(
 			}
 
 			addr = mmap(NULL, sz, PROT_READ | PROT_WRITE,
-				MAP_SHARED | MAP_ANONYMOUS, shm_fd, 0);
+				MAP_PRIVATE | MAP_ANONYMOUS, shm_fd, 0);
 			if (addr == MAP_FAILED) {
 				ok = false;
 				pr_fail_err("mmap");
@@ -188,8 +195,8 @@ static int stress_shm_posix_child(
 			inc_counter(args);
 		}
 reap:
-		for (i = 0; ok && (i < (ssize_t)opt_shm_posix_objects); i++) {
-			char *shm_name = shm_names[i];
+		for (i = 0; ok && (i < (ssize_t)shm_posix_objects); i++) {
+			char *shm_name = &shm_names[i * SHM_NAME_LEN];
 
 			if (addrs[i])
 				(void)munmap(addrs[i], sz);
@@ -202,7 +209,7 @@ reap:
 			/* Inform parent shm ID is now free */
 			msg.index = i;
 			msg.shm_name[SHM_NAME_LEN - 1] = '\0';
-			strncpy(msg.shm_name, shm_name, SHM_NAME_LEN - 1);
+			(void)strncpy(msg.shm_name, shm_name, SHM_NAME_LEN - 1);
 			if (write(fd, &msg, sizeof(msg)) < 0) {
 				pr_dbg("%s: write failed: errno=%d: (%s)\n",
 					args->name, errno, strerror(errno));
@@ -215,12 +222,14 @@ reap:
 
 	/* Inform parent of end of run */
 	msg.index = -1;
-	strncpy(msg.shm_name, "", SHM_NAME_LEN);
+	(void)strncpy(msg.shm_name, "", SHM_NAME_LEN);
 	if (write(fd, &msg, sizeof(msg)) < 0) {
 		pr_err("%s: write failed: errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 	}
+	free(shm_names);
+	free(addrs);
 
 	return rc;
 }
@@ -239,20 +248,28 @@ int stress_shm(const args_t *args)
 	pid_t pid;
 	bool retry = true;
 	uint32_t restarts = 0;
+	size_t shm_posix_bytes = DEFAULT_SHM_POSIX_BYTES;
+	size_t shm_posix_objects = DEFAULT_SHM_POSIX_OBJECTS;
 
-	if (!set_shm_posix_bytes) {
+	if (!get_setting("shm-bytes", &shm_posix_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_shm_posix_bytes = MAX_SHM_POSIX_BYTES;
+			shm_posix_bytes = MAX_SHM_POSIX_BYTES;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_shm_posix_bytes = MIN_SHM_POSIX_BYTES;
+			shm_posix_bytes = MIN_SHM_POSIX_BYTES;
 	}
-	if (!set_shm_posix_objects) {
+	shm_posix_bytes /= args->num_instances;
+	if (shm_posix_bytes < MIN_SHM_POSIX_BYTES)
+		shm_posix_bytes = MIN_SHM_POSIX_BYTES;
+	if (shm_posix_bytes < page_size)
+		shm_posix_bytes = page_size;
+
+	if (!get_setting("shm-objs", &shm_posix_objects)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_shm_posix_objects = MAX_SHM_POSIX_OBJECTS;
+			shm_posix_objects = MAX_SHM_POSIX_OBJECTS;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_shm_posix_objects = MIN_SHM_POSIX_OBJECTS;
+			shm_posix_objects = MIN_SHM_POSIX_OBJECTS;
 	}
-	orig_sz = sz = opt_shm_posix_bytes & ~(page_size - 1);
+	orig_sz = sz = shm_posix_bytes & ~(page_size - 1);
 
 	while (g_keep_stressing_flag && retry) {
 		if (pipe(pipefds) < 0) {
@@ -275,13 +292,17 @@ fork_again:
 		} else if (pid > 0) {
 			/* Parent */
 			int status;
-			char shm_names[MAX_SHM_POSIX_OBJECTS][SHM_NAME_LEN];
+			char *shm_names;
 
+			shm_names = calloc(shm_posix_objects, SHM_NAME_LEN);
+			if (!shm_names) {
+				pr_fail_err("calloc on shm_names");
+				rc = EXIT_NO_RESOURCE;
+				goto err;
+			}
 			(void)setpgid(pid, g_pgrp);
 			(void)close(pipefds[1]);
 
-			memset(shm_names, 0, sizeof(shm_names));
-
 			while (g_keep_stressing_flag) {
 				ssize_t n;
 				shm_msg_t 	msg;
@@ -305,14 +326,14 @@ fork_again:
 					break;
 				}
 				if ((msg.index < 0) ||
-				    (msg.index >= MAX_SHM_POSIX_OBJECTS)) {
+				    (msg.index >= (ssize_t)shm_posix_objects)) {
 					retry = false;
 					break;
 				}
 
-				shm_name = shm_names[msg.index];
+				shm_name = &shm_names[msg.index * SHM_NAME_LEN];
 				shm_name[SHM_NAME_LEN - 1] = '\0';
-				strncpy(shm_name, msg.shm_name, SHM_NAME_LEN - 1);
+				(void)strncpy(shm_name, msg.shm_name, SHM_NAME_LEN - 1);
 			}
 			(void)kill(pid, SIGKILL);
 			(void)waitpid(pid, &status, 0);
@@ -335,18 +356,19 @@ fork_again:
 			 *  has died, so we should be able to remove the
 			 *  shared memory segment.
 			 */
-			for (i = 0; i < (ssize_t)opt_shm_posix_objects; i++) {
-				char *shm_name = shm_names[i];
+			for (i = 0; i < (ssize_t)shm_posix_objects; i++) {
+				char *shm_name = &shm_names[i * SHM_NAME_LEN];
 				if (*shm_name)
 					(void)shm_unlink(shm_name);
 			}
+			free(shm_names);
 		} else if (pid == 0) {
 			/* Child, stress memory */
 			(void)setpgid(0, g_pgrp);
 			stress_parent_died_alarm();
 
 			(void)close(pipefds[0]);
-			rc = stress_shm_posix_child(args, pipefds[1], sz);
+			rc = stress_shm_posix_child(args, pipefds[1], sz, shm_posix_objects);
 			(void)close(pipefds[1]);
 			_exit(rc);
 		}
@@ -358,6 +380,7 @@ fork_again:
 		pr_dbg("%s: OOM restarts: %" PRIu32 "\n",
 			args->name, restarts);
 	}
+err:
 	return rc;
 }
 #else
