@@ -131,7 +131,7 @@ static void *stress_sys_rw_thread(void *ctxt_ptr)
 	 *  Block all signals, let controlling thread
 	 *  handle these
 	 */
-	sigprocmask(SIG_BLOCK, &set, NULL);
+	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 
 	/*
 	 *  According to POSIX.1 a thread should have
@@ -139,7 +139,7 @@ static void *stress_sys_rw_thread(void *ctxt_ptr)
 	 *  However, we block signals in this thread
 	 *  so this is probably just totally unncessary.
 	 */
-	memset(stack, 0, sizeof(stack));
+	(void)memset(stack, 0, sizeof(stack));
 	if (stress_sigaltstack(stack, SIGSTKSZ) < 0)
 		return &nowt;
 
@@ -163,7 +163,7 @@ static void stress_sys_rw_threads(const args_t *args, const char *path)
 	ctxt.args = args;
 	ctxt.path = path;
 
-	memset(ret, 0, sizeof(ret));
+	(void)memset(ret, 0, sizeof(ret));
 
 	ctxt.badbuf = mmap(NULL, SYS_BUF_SZ, PROT_READ,
 			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
@@ -189,7 +189,7 @@ static void stress_sys_rw_threads(const args_t *args, const char *path)
 	}
 
 	if (ctxt.badbuf)
-		munmap(ctxt.badbuf, SYS_BUF_SZ);
+		(void)munmap(ctxt.badbuf, SYS_BUF_SZ);
 }
 
 /*
