@@ -32,19 +32,18 @@ static double rate_ns;
 static double start;
 #endif
 
-static bool set_timer_freq = false;
-static uint64_t opt_timer_freq = DEFAULT_TIMER_FREQ;
-
 /*
  *  stress_set_timer_freq()
  *	set timer frequency from given option
  */
 void stress_set_timer_freq(const char *opt)
 {
-	set_timer_freq = true;
-	opt_timer_freq = get_uint64(opt);
-	check_range("timer-freq", opt_timer_freq,
+	uint64_t timer_freq;
+
+	timer_freq = get_uint64(opt);
+	check_range("timer-freq", timer_freq,
 		MIN_TIMER_FREQ, MAX_TIMER_FREQ);
+	set_setting("timer-freq", TYPE_ID_UINT64, &timer_freq);
 }
 
 #if defined(HAVE_LIB_RT) && defined(__linux__)
@@ -65,8 +64,8 @@ static void stress_timer_set(struct itimerspec *timer)
 		rate = rate_ns;
 	}
 
-	timer->it_value.tv_sec = (long long int)rate / 1000000000;
-	timer->it_value.tv_nsec = (long long int)rate % 1000000000;
+	timer->it_value.tv_sec = (time_t)rate / 1000000000;
+	timer->it_value.tv_nsec = (suseconds_t)rate % 1000000000;
 	if (timer->it_value.tv_sec == 0 &&
 	    timer->it_value.tv_nsec < 1)
 		timer->it_value.tv_nsec = 1;
@@ -106,7 +105,7 @@ static void MLOCKED stress_timer_handler(int sig)
 cancel:
 	g_keep_stressing_flag = false;
 	/* Cancel timer if we detect no more runs */
-	memset(&timer, 0, sizeof(timer));
+	(void)memset(&timer, 0, sizeof(timer));
 	(void)timer_settime(timerid, 0, &timer, NULL);
 }
 
@@ -119,20 +118,21 @@ int stress_timer(const args_t *args)
 	struct sigevent sev;
 	struct itimerspec timer;
 	sigset_t mask;
+	uint64_t timer_freq = DEFAULT_TIMER_FREQ;
 
-	sigemptyset(&mask);
-	sigaddset(&mask, SIGINT);
-	sigprocmask(SIG_SETMASK, &mask, NULL);
+	(void)sigemptyset(&mask);
+	(void)sigaddset(&mask, SIGINT);
+	(void)sigprocmask(SIG_SETMASK, &mask, NULL);
 
 	start = time_now();
 
-	if (!set_timer_freq) {
+	if (!get_setting("timer-freq", &timer_freq)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_timer_freq = MAX_TIMER_FREQ;
+			timer_freq = MAX_TIMER_FREQ;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_timer_freq = MIN_TIMER_FREQ;
+			timer_freq = MIN_TIMER_FREQ;
 	}
-	rate_ns = opt_timer_freq ? 1000000000 / opt_timer_freq : 1000000000;
+	rate_ns = timer_freq ? 1000000000.0 / timer_freq : 1000000000.0;
 
 	if (stress_sighandler(args->name, SIGRTMIN, stress_timer_handler, NULL) < 0)
 		return EXIT_FAILURE;
