@@ -72,7 +72,7 @@ int stress_tlb_shootdown(const args_t *args)
 				break;
 			}
 		}
-		memset(mem, 0, mmap_size);
+		(void)memset(mem, 0, mmap_size);
 
 		for (i = 0; i < tlb_procs; i++)
 			pids[i] = -1;
@@ -104,7 +104,7 @@ int stress_tlb_shootdown(const args_t *args)
 				for (ptr = mem; ptr < mem + mmap_size; ptr += page_size) {
 					/* Force tlb shoot down on page */
 					(void)mprotect(ptr, page_size, PROT_READ);
-					memcpy(buffer, ptr, page_size);
+					(void)memcpy(buffer, ptr, page_size);
 					(void)munmap(ptr, page_size);
 				}
 				_exit(0);
