@@ -42,17 +42,14 @@ typedef struct {
 
 #endif
 
-static size_t opt_userfaultfd_bytes = DEFAULT_MMAP_BYTES;
-static bool set_userfaultfd_bytes = false;
-
 void stress_set_userfaultfd_bytes(const char *opt)
 {
-	set_userfaultfd_bytes = true;
-	opt_userfaultfd_bytes = (size_t)
-		get_uint64_byte_memory(opt,
-			stressor_instances(STRESS_USERFAULTFD));
-	check_range_bytes("userfaultfd-bytes", opt_userfaultfd_bytes,
+	size_t userfaultfd_bytes;
+
+	userfaultfd_bytes = (size_t)get_uint64_byte_memory(opt, 1);
+	check_range_bytes("userfaultfd-bytes", userfaultfd_bytes,
 		MIN_MMAP_BYTES, MAX_MEM_LIMIT);
+	set_setting("userfaultfd-bytes", TYPE_ID_SIZE_T, &userfaultfd_bytes);
 }
 
 #if defined(__linux__) && defined(__NR_userfaultfd)
@@ -150,7 +147,9 @@ static inline int handle_page_fault(
  *	is an OOM-able child process that the
  *	parent can restart
  */
-static int stress_userfaultfd_oomable(const args_t *args)
+static int stress_userfaultfd_oomable(
+	const args_t *args,
+	size_t userfaultfd_bytes)
 {
 	const size_t page_size = args->page_size;
 	size_t sz;
@@ -171,13 +170,7 @@ static int stress_userfaultfd_oomable(const args_t *args)
 		stress_get_stack_direction() * (STACK_SIZE - 64);
 	uint8_t *stack_top = stack + stack_offset;
 
-	if (!set_userfaultfd_bytes) {
-		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_userfaultfd_bytes = MAX_MMAP_BYTES;
-		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_userfaultfd_bytes = MIN_MMAP_BYTES;
-	}
-	sz = opt_userfaultfd_bytes & ~(page_size - 1);
+	sz = userfaultfd_bytes & ~(page_size - 1);
 
 	if (posix_memalign(&zero_page, page_size, page_size)) {
 		pr_err("%s: zero page allocation failed\n", args->name);
@@ -204,7 +197,7 @@ static int stress_userfaultfd_oomable(const args_t *args)
 		do_poll = false;
 
 	/* API sanity check */
-	memset(&api, 0, sizeof(api));
+	(void)memset(&api, 0, sizeof(api));
 	api.api = UFFD_API;
 	api.features = 0;
 	if (ioctl(fd, UFFDIO_API, &api) < 0) {
@@ -221,7 +214,7 @@ static int stress_userfaultfd_oomable(const args_t *args)
 	}
 
 	/* Register fault handling mode */
-	memset(&reg, 0, sizeof(reg));
+	(void)memset(&reg, 0, sizeof(reg));
 	reg.range.start = (unsigned long)data;
 	reg.range.len = sz;
 	reg.mode = UFFDIO_REGISTER_MODE_MISSING;
@@ -282,7 +275,7 @@ static int stress_userfaultfd_oomable(const args_t *args)
 		if (do_poll) {
 			struct pollfd fds[1];
 
-			memset(fds, 0, sizeof fds);
+			(void)memset(fds, 0, sizeof fds);
 			fds[0].fd = fd;
 			fds[0].events = POLLIN;
 			/* wait for 1 second max */
@@ -364,6 +357,19 @@ int stress_userfaultfd(const args_t *args)
 {
 	pid_t pid;
 	int rc = EXIT_FAILURE;
+	size_t userfaultfd_bytes = DEFAULT_MMAP_BYTES;
+
+	if (!get_setting("userfaultfd-bytes", &userfaultfd_bytes)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			userfaultfd_bytes = MAX_MMAP_BYTES;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			userfaultfd_bytes = MIN_MMAP_BYTES;
+	}
+	userfaultfd_bytes /= args->num_instances;
+	if (userfaultfd_bytes < MIN_MMAP_BYTES)
+		userfaultfd_bytes = MIN_MMAP_BYTES;
+	if (userfaultfd_bytes < args->page_size)
+		userfaultfd_bytes = args->page_size;
 
 	pid = fork();
 	if (pid < 0) {
@@ -405,7 +411,7 @@ int stress_userfaultfd(const args_t *args)
 		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 
-		_exit(stress_userfaultfd_oomable(args));
+		_exit(stress_userfaultfd_oomable(args, userfaultfd_bytes));
 	}
 	return rc;
 }
