@@ -43,36 +43,97 @@ typedef size_t (*stress_vm_func)(uint8_t *buf, const size_t sz,
 typedef struct {
 	const char *name;
 	const stress_vm_func func;
-} stress_vm_stressor_info_t;
+} stress_vm_method_info_t;
 
-static uint64_t opt_vm_hang = DEFAULT_VM_HANG;
-static size_t   opt_vm_bytes = DEFAULT_VM_BYTES;
-static bool	set_vm_bytes = false;
-static int      opt_vm_flags = 0;                      /* VM mmap flags */
+typedef struct {
+	const char *name;
+        int advice;
+} vm_madvise_info_t;
 
-static const stress_vm_stressor_info_t *opt_vm_stressor;
-static const stress_vm_stressor_info_t vm_methods[];
+static const stress_vm_method_info_t vm_methods[];
+
+static const vm_madvise_info_t vm_madvise_info[] = {
+#if defined(HAVE_MADVISE)
+#if defined(MADV_DONTNEED)
+	{ "dontneed",	MADV_DONTNEED},
+#endif
+#if defined(MADV_HUGEPAGE)
+	{ "hugepage",	MADV_HUGEPAGE },
+#endif
+#if defined(MADV_MERGEABLE)
+	{ "mergeable",	MADV_MERGEABLE },
+#endif
+#if defined(MADV_NOHUGEPAGE)
+	{ "nohugepage",	MADV_NOHUGEPAGE },
+#endif
+#if defined(MADV_NORMAL)
+	{ "normal",	MADV_NORMAL },
+#endif
+#if defined(MADV_RANDOM)
+	{ "random",	MADV_RANDOM },
+#endif
+#if defined(MADV_SEQUENTIAL)
+	{ "sequential",	MADV_SEQUENTIAL },
+#endif
+#if defined(MADV_UNMERGEABLE)
+	{ "unmergeable",MADV_UNMERGEABLE },
+#endif
+#if defined(MADV_WILLNEED)
+	{ "willneed",	MADV_WILLNEED},
+#endif
+        { NULL,         0 },
+#else
+	/* No MADVISE, default to normal, ignored */
+	{ "normal",	0 },
+#endif
+};
 
 void stress_set_vm_hang(const char *opt)
 {
-	opt_vm_hang = get_uint64_byte(opt);
-	check_range("vm-hang", opt_vm_hang,
+	uint64_t vm_hang;
+
+	vm_hang = get_uint64_time(opt);
+	check_range("vm-hang", vm_hang,
 		MIN_VM_HANG, MAX_VM_HANG);
+	set_setting("vm-hang", TYPE_ID_UINT64, &vm_hang);
 }
 
 void stress_set_vm_bytes(const char *opt)
 {
-	set_vm_bytes = true;
-	opt_vm_bytes = (size_t)
-		get_uint64_byte_memory(opt,
-			stressor_instances(STRESS_VM));
-	check_range_bytes("vm-bytes", opt_vm_bytes,
+	size_t vm_bytes;
+
+	vm_bytes = (size_t)get_uint64_byte_memory(opt, 1);
+	check_range_bytes("vm-bytes", vm_bytes,
 		MIN_VM_BYTES, MAX_MEM_LIMIT);
+	set_setting("vm-bytes", TYPE_ID_SIZE_T, &vm_bytes);
 }
 
 void stress_set_vm_flags(const int flag)
 {
-	opt_vm_flags |= flag;
+	int vm_flags = 0;
+
+	(void)get_setting("vm-flags", &vm_flags);
+	vm_flags |= flag;
+	set_setting("vm-flags", TYPE_ID_INT, &vm_flags);
+
+}
+
+int stress_set_vm_madvise(const char *opt)
+{
+	const vm_madvise_info_t *info;
+
+	for (info = vm_madvise_info; info->name; info++) {
+		if (!strcmp(opt, info->name)) {
+			set_setting("vm-madvise", TYPE_ID_INT, &info->advice);
+			return 0;
+		}
+	}
+	fprintf(stderr, "invalid vm-madvise advice '%s', allowed advice options are:", opt);
+	for (info = vm_madvise_info; info->name; info++) {
+		fprintf(stderr, " %s", info->name);
+        }
+	fprintf(stderr, "\n");
+	return -1;
 }
 
 #define SET_AND_TEST(ptr, val, bit_errors)	\
@@ -446,7 +507,7 @@ static size_t stress_vm_walking_one_addr(
 	size_t tests = 0;
 	uint64_t c = *counter;
 
-	memset(buf, d1, sz);
+	(void)memset(buf, d1, sz);
 	for (ptr = buf; ptr < buf_end; ptr += 256) {
 		uint16_t i;
 		uint64_t mask;
@@ -502,7 +563,7 @@ static size_t stress_vm_walking_zero_addr(
 
 	sz_mask--;
 
-	memset(buf, d1, sz);
+	(void)memset(buf, d1, sz);
 	for (ptr = buf; ptr < buf_end; ptr += 256) {
 		uint16_t i;
 		uint64_t mask;
@@ -594,7 +655,7 @@ static size_t stress_vm_incdec(
 	uint64_t c = *counter;
 
 	val++;
-	memset(buf, 0x00, sz);
+	(void)memset(buf, 0x00, sz);
 
 	for (ptr = buf; ptr < buf_end; ptr++) {
 		*ptr += val;
@@ -647,7 +708,7 @@ static size_t stress_vm_prime_incdec(
 		return 0;
 #endif
 
-	memset(buf, 0x00, sz);
+	(void)memset(buf, 0x00, sz);
 
 	for (i = 0; i < sz; i++) {
 		ptr[i] += val;
@@ -714,7 +775,7 @@ static size_t stress_vm_swap(
 	mwc_seed(w1, z1);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
 		uint8_t val = mwc8();
-		memset((void *)ptr, val, chunk_sz);
+		(void)memset((void *)ptr, val, chunk_sz);
 	}
 
 	/* Forward swaps */
@@ -1051,7 +1112,7 @@ static size_t stress_vm_zero_one(
 
 	(void)max_ops;
 
-	memset(buf, 0x00, sz);
+	(void)memset(buf, 0x00, sz);
 	(void)mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 	c += sz / 8;
@@ -1070,7 +1131,7 @@ static size_t stress_vm_zero_one(
 			goto abort;
 	}
 
-	memset(buf, 0xff, sz);
+	(void)memset(buf, 0xff, sz);
 	(void)mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 	c += sz / 8;
@@ -1113,7 +1174,7 @@ static size_t stress_vm_galpat_zero(
 	size_t bits_bad = sz / 4096;
 	uint64_t c = *counter;
 
-	memset(buf, 0x00, sz);
+	(void)memset(buf, 0x00, sz);
 
 	mwc_reseed();
 
@@ -1175,7 +1236,7 @@ static size_t stress_vm_galpat_one(
 	size_t bits_bad = sz / 4096;
 	uint64_t c = *counter;
 
-	memset(buf, 0xff, sz);
+	(void)memset(buf, 0xff, sz);
 
 	mwc_reseed();
 
@@ -1235,7 +1296,7 @@ static size_t stress_vm_inc_nybble(
 	size_t bit_errors = 0;
 	uint64_t c = *counter;
 
-	memset(buf, val, sz);
+	(void)memset(buf, val, sz);
 	INC_LO_NYBBLE(val);
 	INC_HI_NYBBLE(val);
 
@@ -1380,7 +1441,7 @@ static size_t stress_vm_prime_zero(
 		return 0;
 #endif
 
-	memset(buf, 0xff, sz);
+	(void)memset(buf, 0xff, sz);
 
 	for (j = 0; j < 8; j++) {
 		uint8_t mask = ~(1 << j);
@@ -1437,7 +1498,7 @@ static size_t stress_vm_prime_one(
 		return 0;
 #endif
 
-	memset(buf, 0x00, sz);
+	(void)memset(buf, 0x00, sz);
 
 	for (j = 0; j < 8; j++) {
 		uint8_t mask = 1 << j;
@@ -1494,7 +1555,7 @@ static size_t stress_vm_prime_gray_zero(
 		return 0;
 #endif
 
-	memset(buf, 0xff, sz);
+	(void)memset(buf, 0xff, sz);
 
 	for (i = 0, j = prime; i < sz; i++, j += prime) {
 		/*
@@ -1561,7 +1622,7 @@ static size_t stress_vm_prime_gray_one(
 		return 0;
 #endif
 
-	memset(buf, 0x00, sz);
+	(void)memset(buf, 0x00, sz);
 
 	for (i = 0, j = prime; i < sz; i++, j += prime) {
 		/*
@@ -1816,7 +1877,7 @@ static size_t stress_vm_all(
 	return bit_errors;
 }
 
-static const stress_vm_stressor_info_t vm_methods[] = {
+static const stress_vm_method_info_t vm_methods[] = {
 	{ "all",	stress_vm_all },
 	{ "flip",	stress_vm_flip },
 	{ "galpat-0",	stress_vm_galpat_zero },
@@ -1852,11 +1913,11 @@ static const stress_vm_stressor_info_t vm_methods[] = {
  */
 int stress_set_vm_method(const char *name)
 {
-	stress_vm_stressor_info_t const *info;
+	stress_vm_method_info_t const *info;
 
 	for (info = vm_methods; info->func; info++) {
 		if (!strcmp(info->name, name)) {
-			opt_vm_stressor = info;
+			set_setting("vm-method", TYPE_ID_UINTPTR_T, &info);
 			return 0;
 		}
 	}
@@ -1878,22 +1939,38 @@ int stress_set_vm_method(const char *name)
 int stress_vm(const args_t *args)
 {
 	uint64_t *bit_error_count = MAP_FAILED;
+	uint64_t vm_hang = DEFAULT_VM_HANG;
 	uint32_t restarts = 0, nomems = 0;
+	size_t vm_bytes = DEFAULT_VM_BYTES;
 	uint8_t *buf = NULL;
 	pid_t pid;
 	const bool keep = (g_opt_flags & OPT_FLAGS_VM_KEEP);
-	const stress_vm_func func = opt_vm_stressor->func;
         const size_t page_size = args->page_size;
 	size_t buf_sz, retries;
 	int err = 0, ret = EXIT_SUCCESS;
+	int vm_flags = 0;                      /* VM mmap flags */
+	int vm_madvise = -1;
+	const stress_vm_method_info_t *vm_method = &vm_methods[0];
+	stress_vm_func func;
 
-	if (!set_vm_bytes) {
+	(void)get_setting("vm-hang", &vm_hang);
+	(void)get_setting("vm-flags", &vm_flags);
+	(void)get_setting("vm-method", &vm_method);
+	(void)get_setting("vm-madvise", &vm_madvise);
+
+	func = vm_method->func;
+	pr_dbg("%s using method '%s'\n", args->name, vm_method->name);
+
+	if (!get_setting("vm-bytes", &vm_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			opt_vm_bytes = MAX_VM_BYTES;
+			vm_bytes = MAX_VM_BYTES;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			opt_vm_bytes = MIN_VM_BYTES;
+			vm_bytes = MIN_VM_BYTES;
 	}
-	buf_sz = opt_vm_bytes & ~(page_size - 1);
+	vm_bytes /= args->num_instances;
+	if (vm_bytes < MIN_VM_BYTES)
+		vm_bytes = MIN_VM_BYTES;
+	buf_sz = vm_bytes & ~(page_size - 1);
 
 	for (retries = 0; (retries < 100) && g_keep_stressing_flag; retries++) {
 		bit_error_count = (uint64_t *)
@@ -1973,15 +2050,18 @@ again:
 					return EXIT_SUCCESS;
 				buf = (uint8_t *)mmap(NULL, buf_sz,
 					PROT_READ | PROT_WRITE,
-					MAP_SHARED | MAP_ANONYMOUS |
-					opt_vm_flags, -1, 0);
+					MAP_PRIVATE | MAP_ANONYMOUS |
+					vm_flags, -1, 0);
 				if (buf == MAP_FAILED) {
 					buf = NULL;
 					no_mem_retries++;
 					(void)shim_usleep(100000);
 					continue;	/* Try again */
 				}
-				(void)madvise_random(buf, buf_sz);
+				if (vm_madvise < 0)
+					(void)madvise_random(buf, buf_sz);
+				else
+					(void)shim_madvise(buf, buf_sz, vm_madvise);
 			}
 
 			no_mem_retries = 0;
@@ -1989,12 +2069,12 @@ again:
 			*bit_error_count += func(buf, buf_sz, args->counter,
 						args->max_ops << VM_BOGO_SHIFT);
 
-			if (opt_vm_hang == 0) {
+			if (vm_hang == 0) {
 				for (;;) {
 					(void)sleep(3600);
 				}
-			} else if (opt_vm_hang != DEFAULT_VM_HANG) {
-				(void)sleep((int)opt_vm_hang);
+			} else if (vm_hang != DEFAULT_VM_HANG) {
+				(void)sleep((int)vm_hang);
 			}
 
 			if (!keep) {
@@ -2014,7 +2094,7 @@ clean_up:
 			args->name, *bit_error_count);
 		ret = EXIT_FAILURE;
 	}
-	(void)munmap(bit_error_count, page_size);
+	(void)munmap((void *)bit_error_count, page_size);
 
 	*args->counter >>= VM_BOGO_SHIFT;
 
