@@ -60,12 +60,11 @@ typedef void (*stress_wcs_func)(
 
 typedef struct {
 	const char		*name;	/* human readable form of stressor */
-	const stress_wcs_func	func;	/* the stressor function */
+	const stress_wcs_func	func;	/* the wcs method function */
 	const void		*libc_func;
-} stress_wcs_stressor_info_t;
+} stress_wcs_method_info_t;
 
-static const stress_wcs_stressor_info_t *opt_wcs_stressor;
-static const stress_wcs_stressor_info_t wcs_methods[];
+static const stress_wcs_method_info_t wcs_methods[];
 
 /*
  *  stress_wcs_fill
@@ -560,7 +559,7 @@ static void stress_wcs_all(
 /*
  * Table of wcs stress methods
  */
-static const stress_wcs_stressor_info_t wcs_methods[] = {
+static const stress_wcs_method_info_t wcs_methods[] = {
 	{ "all",		stress_wcs_all,		NULL },	/* Special "all" test */
 
 #if defined(WCSCASECMP)
@@ -596,19 +595,18 @@ static const stress_wcs_stressor_info_t wcs_methods[] = {
  */
 int stress_set_wcs_method(const char *name)
 {
-	stress_wcs_stressor_info_t const *wcsfunction = wcs_methods;
+	stress_wcs_method_info_t const *info;
 
-
-	for (wcsfunction = wcs_methods; wcsfunction->func; wcsfunction++) {
-		if (!strcmp(wcsfunction->name, name)) {
-			opt_wcs_stressor = wcsfunction;
+	for (info = wcs_methods; info->func; info++) {
+		if (!strcmp(info->name, name)) {
+			set_setting("wcs-method", TYPE_ID_UINTPTR_T, &info);
 			return 0;
 		}
 	}
 
 	(void)fprintf(stderr, "wcs-method must be one of:");
-	for (wcsfunction = wcs_methods; wcsfunction->func; wcsfunction++) {
-		(void)fprintf(stderr, " %s", wcsfunction->name);
+	for (info = wcs_methods; info->func; info++) {
+		(void)fprintf(stderr, " %s", info->name);
 	}
 	(void)fprintf(stderr, "\n");
 
@@ -621,10 +619,15 @@ int stress_set_wcs_method(const char *name)
  */
 int stress_wcs(const args_t *args)
 {
-	stress_wcs_func func = opt_wcs_stressor->func;
-	const void *libc_func = opt_wcs_stressor->libc_func;
+	stress_wcs_method_info_t const *wcs_method = &wcs_methods[0];
+	stress_wcs_func func;
+	const void *libc_func;
 	bool failed = false;
 
+	(void)get_setting("wcs-method", &wcs_method);
+	func = wcs_method->func;
+	libc_func = wcs_method->libc_func;
+
 	do {
 		wchar_t str1[STR1LEN], str2[STR2LEN];
 
