@@ -32,26 +32,20 @@
  *  stress_xattr
  *	stress the xattr operations
  */
-int stress_xattr(
-	uint64_t *const counter,
-	const uint32_t instance,
-	const uint64_t max_ops,
-	const char *name)
+int stress_xattr(const args_t *args)
 {
-	pid_t pid = getpid();
 	int ret, fd, rc = EXIT_FAILURE;
 	char filename[PATH_MAX];
 
-	ret = stress_temp_dir_mk(name, pid, instance);
+	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
 		return exit_status(-ret);
 
-	(void)stress_temp_filename(filename, sizeof(filename),
-		name, pid, instance, mwc32());
+	(void)stress_temp_filename_args(args, filename, sizeof(filename), mwc32());
 	(void)umask(0077);
 	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
 		rc = exit_status(errno);
-		pr_fail_err(name, "open");
+		pr_fail_err("open");
 		goto out;
 	}
 	(void)unlink(filename);
@@ -64,57 +58,57 @@ int stress_xattr(
 		char *buffer;
 
 		for (i = 0; i < 4096; i++) {
-			snprintf(attrname, sizeof(attrname), "user.var_%d", i);
-			snprintf(value, sizeof(value), "orig-value-%d", i);
+			(void)snprintf(attrname, sizeof(attrname), "user.var_%d", i);
+			(void)snprintf(value, sizeof(value), "orig-value-%d", i);
 
 			ret = fsetxattr(fd, attrname, value, strlen(value), XATTR_CREATE);
 			if (ret < 0) {
 				if (errno == ENOTSUP) {
-					pr_inf(stdout, "%s stressor will be "
+					pr_inf("%s stressor will be "
 						"skipped, filesystem does not "
-						"support xattr.\n", name);
+						"support xattr.\n", args->name);
 				}
 				if (errno == ENOSPC || errno == EDQUOT)
 					break;
-				pr_fail_err(name, "fsetxattr");
+				pr_fail_err("fsetxattr");
 				goto out_close;
 			}
 		}
 		for (j = 0; j < i; j++) {
-			snprintf(attrname, sizeof(attrname), "user.var_%d", j);
-			snprintf(value, sizeof(value), "value-%d", j);
+			(void)snprintf(attrname, sizeof(attrname), "user.var_%d", j);
+			(void)snprintf(value, sizeof(value), "value-%d", j);
 
 			ret = fsetxattr(fd, attrname, value, strlen(value),
 				XATTR_REPLACE);
 			if (ret < 0) {
 				if (errno == ENOSPC || errno == EDQUOT)
 					break;
-				pr_fail_err(name, "fsetxattr");
+				pr_fail_err("fsetxattr");
 				goto out_close;
 			}
 		}
 		for (j = 0; j < i; j++) {
 			char tmp[sizeof(value)];
 
-			snprintf(attrname, sizeof(attrname), "user.var_%d", j);
-			snprintf(value, sizeof(value), "value-%d", j);
+			(void)snprintf(attrname, sizeof(attrname), "user.var_%d", j);
+			(void)snprintf(value, sizeof(value), "value-%d", j);
 
 			ret = fgetxattr(fd, attrname, tmp, sizeof(tmp));
 			if (ret < 0) {
-				pr_fail_err(name, "fgetxattr");
+				pr_fail_err("fgetxattr");
 				goto out_close;
 			}
 			if (strncmp(value, tmp, ret)) {
-				pr_fail(stderr, "%s: fgetxattr values "
+				pr_fail("%s: fgetxattr values "
 					"different %.*s vs %.*s\n",
-					name, ret, value, ret, tmp);
+					args->name, ret, value, ret, tmp);
 				goto out_close;
 			}
 		}
 		/* Determine how large a buffer we required... */
 		sz = flistxattr(fd, NULL, 0);
 		if (sz < 0) {
-			pr_fail_err(name, "flistxattr");
+			pr_fail_err("flistxattr");
 			goto out_close;
 		}
 		buffer = malloc(sz);
@@ -124,36 +118,32 @@ int stress_xattr(
 			free(buffer);
 
 			if (sz < 0) {
-				pr_fail_err(name, "flistxattr");
+				pr_fail_err("flistxattr");
 				goto out_close;
 			}
 		}
 		for (j = 0; j < i; j++) {
-			snprintf(attrname, sizeof(attrname), "user.var_%d", j);
+			(void)snprintf(attrname, sizeof(attrname), "user.var_%d", j);
 
 			ret = fremovexattr(fd, attrname);
 			if (ret < 0) {
-				pr_fail_err(name, "fremovexattr");
+				pr_fail_err("fremovexattr");
 				goto out_close;
 			}
 		}
-		(*counter)++;
-	} while (opt_do_run && (!max_ops || *counter < max_ops));
+		inc_counter(args);
+	} while (keep_stressing());
 
 	rc = EXIT_SUCCESS;
 out_close:
 	(void)close(fd);
 out:
-	(void)stress_temp_dir_rm(name, pid, instance);
+	(void)stress_temp_dir_rm_args(args);
 	return rc;
 }
 #else
-int stress_xattr(
-	uint64_t *const counter,
-	const uint32_t instance,
-	const uint64_t max_ops,
-	const char *name)
+int stress_xattr(const args_t *args)
 {
-	return stress_not_implemented(counter, instance, max_ops, name);
+	return stress_not_implemented(args);
 }
 #endif
