@@ -36,7 +36,7 @@ int stress_yield(const args_t *args)
 	uint64_t max_ops_per_yielder;
 	size_t counters_sz, yielders_sz;
 	int32_t cpus = stress_get_processors_configured();
-	size_t instances = stressor_instances(STRESS_YIELD);
+	const size_t instances = args->num_instances;
 	size_t yielders = 2;
 #if defined(__linux__) && NEED_GLIBC(2,3,0)
 	cpu_set_t mask;
@@ -66,10 +66,6 @@ int stress_yield(const args_t *args)
 	 *  CPU available to force context switching on yields
 	 */
 	if (cpus > 0) {
-		if (!instances) {
-			/* Should not happen, but avoids division by zero */
-			instances = 1;
-		}
 		cpus *= 2;
 		yielders = cpus / instances;
 		if (yielders < 1)
@@ -88,7 +84,7 @@ int stress_yield(const args_t *args)
 		pr_err("%s: calloc failed\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
-	memset(pids, 0, yielders_sz);
+	(void)memset(pids, 0, yielders_sz);
 
 	counters_sz = yielders * sizeof(uint64_t);
 	counters = (uint64_t *)mmap(NULL, counters_sz, PROT_READ | PROT_WRITE,
@@ -101,7 +97,7 @@ int stress_yield(const args_t *args)
 		free(pids);
 		return rc;
 	}
-	memset(counters, 0, counters_sz);
+	(void)memset(counters, 0, counters_sz);
 
 	for (i = 0; g_keep_stressing_flag && (i < yielders); i++) {
 		pids[i] = fork();
@@ -127,7 +123,7 @@ int stress_yield(const args_t *args)
 
 	do {
 		*args->counter = 0;
-		pause();
+		(void)pause();
 		for (i = 0; i < yielders; i++)
 			*args->counter += counters[i];
 	} while (keep_stressing());
