@@ -28,19 +28,17 @@
  *  stress_zero
  *	stress reading of /dev/zero
  */
-int stress_zero(
-	uint64_t *const counter,
-	const uint32_t instance,
-	const uint64_t max_ops,
-	const char *name)
+int stress_zero(const args_t *args)
 {
 	int fd;
-	const size_t page_size = stress_get_pagesize();
-
-	(void)instance;
-
-	if ((fd = open("/dev/zero", O_RDONLY)) < 0) {
-		pr_fail_err(name, "open");
+	const size_t page_size = args->page_size;
+#if defined(__minix__)
+	const int flags = O_RDONLY;
+#else
+	const int flags = O_RDWR;
+#endif
+	if ((fd = open("/dev/zero", flags)) < 0) {
+		pr_fail_err("open /dev/zero");
 		return EXIT_FAILURE;
 	}
 
@@ -55,36 +53,47 @@ int stress_zero(
 		if (ret < 0) {
 			if ((errno == EAGAIN) || (errno == EINTR))
 				continue;
-			pr_fail_err(name, "read");
+			pr_fail_err("read");
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
 
+#if !defined(__minix__)
+		/* One can also write to /dev/zero w/o failure */
+		ret = write(fd, buffer, sizeof(buffer));
+		if (ret < 0) {
+			if ((errno == EAGAIN) || (errno == EINTR))
+				continue;
+			pr_fail_err("write");
+			(void)close(fd);
+			return EXIT_FAILURE;
+		}
+#endif
+
 #if defined(__linux__)
 		/*
 		 *  check if we can mmap /dev/zero
 		 */
-		ptr = mmap(NULL, page_size, PROT_READ, MAP_SHARED | MAP_ANONYMOUS,
+		ptr = mmap(NULL, page_size, PROT_READ, MAP_PRIVATE | MAP_ANONYMOUS,
 			fd, page_size * mwc16());
 		if (ptr == MAP_FAILED) {
 			if (errno == ENOMEM)
 				continue;
-			pr_fail_err(name, "mmap /dev/zero");
+			pr_fail_err("mmap /dev/zero");
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
 		/* Quick sanity check if first 32 bits are zero */
 		if (*ptr != 0) {
-			pr_fail_err(name, "mmap'd /dev/zero not null");
+			pr_fail_err("mmap'd /dev/zero not null");
 			(void)munmap(ptr, page_size);
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
 		(void)munmap(ptr, page_size);
 #endif
-
-		(*counter)++;
-	} while (opt_do_run && (!max_ops || *counter < max_ops));
+		inc_counter(args);
+	} while (keep_stressing());
 	(void)close(fd);
 
 	return EXIT_SUCCESS;
