@@ -43,10 +43,52 @@ typedef struct {
 	const stress_zlib_rand_data_func func;	/* the random data generation function */
 } stress_zlib_rand_data_info_t;
 
-static const stress_zlib_rand_data_info_t *opt_zlib_rand_data_func;
-static const stress_zlib_rand_data_info_t zlib_rand_data_methods[];
+static stress_zlib_rand_data_info_t zlib_rand_data_methods[];
 static volatile bool pipe_broken = false;
 
+static const char *const lorem_ipsum[] = {
+	"Lorem ipsum dolor sit amet, consectetur adipiscing elit. ",
+	"Nullam imperdiet quam at ultricies finibus. ",
+	"Nunc venenatis euismod velit sit amet ornare.",
+	"Quisque et orci eu eros convallis luctus at facilisis ex. ",
+	"Quisque fringilla nulla felis, sed mollis est feugiat nec. ",
+	"Vivamus at urna sit amet velit suscipit iaculis. ",
+	"Curabitur mauris ipsum, gravida in laoreet ac, dignissim id lacus. ",
+	"Proin dignissim, erat nec interdum commodo, nulla mi tempor dui, quis scelerisque odio nisi in tortor. ",
+	"Mauris dignissim ex auctor nulla lobortis semper. ",
+	"Mauris sit amet tempus risus, ac tincidunt lectus. ",
+	"Maecenas sollicitudin porttitor nisi ac faucibus. ",
+	"Cras eu sollicitudin arcu. ",
+	"In sed fringilla eros, vitae fringilla tortor. ",
+	"Phasellus mollis feugiat tortor, a ornare nunc auctor porttitor. ",
+	"Fusce malesuada ut felis vitae vestibulum. ",
+	"Donec sit amet hendrerit massa, vitae ultrices augue. ",
+	"Proin volutpat velit ipsum, id imperdiet risus placerat ut. ",
+	"Cras vitae risus ipsum.  ",
+	"Sed lobortis quam in dictum pulvinar. ",
+	"In non accumsan justo. ",
+	"Ut pretium pulvinar gravida. ",
+	"Proin ultricies nisi libero, a convallis dui vestibulum eu. ",
+	"Aliquam in molestie magna, et ullamcorper turpis. ",
+	"Donec id pharetra sem.  "
+	"Duis dui massa, fringilla id mattis nec, consequat eget felis. ",
+	"Integer a lobortis ipsum, quis ornare felis. ",
+	"Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. ",
+	"Nulla sed cursus nibh. ",
+	"Quisque at ex dolor. ",
+	"Mauris viverra risus pellentesque nisl dictum rutrum. ",
+	"Aliquam non est quis enim dictum tristique. ",
+	"Fusce feugiat hendrerit hendrerit. ",
+	"Ut egestas sed erat et egestas. ",
+	"Pellentesque convallis erat sed sapien pellentesque vulputate. ",
+	"Praesent non sapien aliquet risus varius suscipit. ",
+	"Curabitur eu felis dignissim, hendrerit magna vitae, condimentum nunc. ",
+	"Donec ut tincidunt sem. ",
+	"Sed in leo et metus ultricies semper quis quis ex. ",
+	"Sed fringilla porta mi vitae condimentum. ",
+	"In vitae metus libero."
+};
+
 /*
  *  stress_sigpipe_handler()
  *      SIGFPE handler
@@ -203,7 +245,7 @@ static void stress_rand_data_rarely_0(uint32_t *data, const int size)
 }
 
 /*
- *  stress_fixed_data()
+ *  stress_rand_data_fixed()
  *	fill buffer with data that is 0x04030201
  */
 static void stress_rand_data_fixed(uint32_t *data, const int size)
@@ -215,6 +257,66 @@ static void stress_rand_data_fixed(uint32_t *data, const int size)
 		*data = 0x04030201;
 }
 
+/*
+ *  stress_rand_data_latin()
+ *	fill buffer with random latin Lorum Ipsum text.
+ */
+static void stress_rand_data_latin(uint32_t *data, const int size)
+{
+	register int i;
+	static const char *ptr = NULL;
+	char *dataptr = (char *)data;
+
+	if (!ptr)
+		ptr = lorem_ipsum[mwc32() % SIZEOF_ARRAY(lorem_ipsum)];
+
+	for (i = 0; i < size; i++) {
+		if (!*ptr)
+			ptr = lorem_ipsum[mwc32() % SIZEOF_ARRAY(lorem_ipsum)];
+
+		*dataptr++ = *ptr++;
+	}
+}
+
+
+#if !defined(__sun__)
+/*
+ *  stress_rand_data_objcode()
+ *	fill buffer with object code data from stress-ng
+ */
+static void stress_rand_data_objcode(uint32_t *data, const int size)
+{
+	const int n = size / sizeof(uint32_t);
+	register int i;
+
+#if defined(__OpenBSD__)
+	extern char _start[];
+#else
+	extern char __executable_start[];
+#endif
+	extern char __etext;
+
+#if defined(__OpenBSD__)
+	char *text_start = &_start[0];
+#else
+	char *text_start = &__executable_start[0];
+#endif
+	char *text_end = &__etext;
+	char *text;
+	const size_t text_len = text_end - text_start;
+
+	/* Start in random place in stress-ng text segment */
+	text = text_start + (mwc64() % text_len);
+
+	for (i = 0; i < n; i++, data++) {
+		*data = *text;
+		text++;
+		if (text >= text_end)
+			text = text_start;
+	}
+}
+#endif
+
 static const stress_zlib_rand_data_func rand_data_funcs[] = {
 	stress_rand_data_rarely_1,
 	stress_rand_data_rarely_0,
@@ -224,7 +326,11 @@ static const stress_zlib_rand_data_func rand_data_funcs[] = {
 	stress_rand_data_digits,
 	stress_rand_data_00_ff,
 	stress_rand_data_nybble,
-	stress_rand_data_fixed
+	stress_rand_data_fixed,
+	stress_rand_data_latin,
+#if !defined(__sun__)
+	stress_rand_data_objcode
+#endif
 };
 
 /*
@@ -240,33 +346,36 @@ static HOT OPTIMIZE3 void stress_zlib_random_test(uint32_t *data, const int size
 /*
  * Table of zlib data methods
  */
-static const stress_zlib_rand_data_info_t zlib_rand_data_methods[] = {
+static stress_zlib_rand_data_info_t zlib_rand_data_methods[] = {
 	{ "random",	stress_zlib_random_test }, /* Special "random" test */
 
-	{ "binary",	stress_rand_data_binary },
-	{ "text",	stress_rand_data_text },
+	{ "00ff",	stress_rand_data_00_ff },
 	{ "ascii01",	stress_rand_data_01 },
 	{ "asciidigits",stress_rand_data_digits },
-	{ "00ff",	stress_rand_data_00_ff },
+	{ "binary",	stress_rand_data_binary },
+	{ "fixed",	stress_rand_data_fixed },
+	{ "latin",	stress_rand_data_latin },
 	{ "nybble",	stress_rand_data_nybble },
+#if !defined(__sun__)
+	{ "objcode",	stress_rand_data_objcode },
+#endif
 	{ "rarely1",	stress_rand_data_rarely_1 },
 	{ "rarely0",	stress_rand_data_rarely_0 },
-	{ "fixed",	stress_rand_data_fixed },
+	{ "text",	stress_rand_data_text },
 	{ NULL,		NULL }
 };
 
-
 /*
  *  stress_set_zlib_method()
  *	set the default zlib random data method
  */
 int HOT OPTIMIZE3 stress_set_zlib_method(const char *name)
 {
-	stress_zlib_rand_data_info_t const *info = zlib_rand_data_methods;
+	stress_zlib_rand_data_info_t *info;
 
 	for (info = zlib_rand_data_methods; info->func; info++) {
 		if (!strcmp(info->name, name)) {
-			opt_zlib_rand_data_func = info;
+			set_setting("zlib-method", TYPE_ID_UINTPTR_T, &info);
 			return 0;
 		}
 	}
@@ -422,6 +531,9 @@ static int stress_zlib_deflate(
 	uint64_t bytes_in = 0, bytes_out = 0, xsum = 0;
 	uint64_t xsum_chars = 0;
 	int flush = Z_FINISH;
+	stress_zlib_rand_data_info_t *opt_zlib_rand_data_func = &zlib_rand_data_methods[0];
+
+	(void)get_setting("zlib-method", &opt_zlib_rand_data_func);
 
 	stream_def.zalloc = Z_NULL;
 	stream_def.zfree = Z_NULL;
